@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximain, Inc. (www.ximian.com)
+ *  Copyright 2001 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  *
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -38,7 +39,9 @@ struct raw_header {
 };
 
 struct _GMimeHeader {
+	GMimeHeaderWriteFunc default_write_func;
 	GHashTable *hash;
+	GHashTable *write_hash;
 	struct raw_header *headers;
 };
 
@@ -62,6 +65,18 @@ header_hash (gconstpointer key)
 	return h;
 }
 
+static gssize
+write_default (GMimeStream *stream, const char *name, const char *value)
+{
+	gssize nwritten;
+	char *val;
+	
+	val = g_mime_utils_header_printf ("%s: %s\n", name, value);
+	nwritten = g_mime_stream_write_string (stream, val);
+	g_free (val);
+	
+	return nwritten;
+}
 
 /**
  * g_mime_header_new:
@@ -76,13 +91,22 @@ g_mime_header_new ()
 	GMimeHeader *new;
 	
 	new = g_new (GMimeHeader, 1);
+	new->default_write_func = write_default;
 	new->hash = g_hash_table_new (header_hash, header_equal);
+	new->write_hash = g_hash_table_new (header_hash, header_equal);
 	new->headers = NULL;
 	
 	return new;
 }
 
 
+static void
+writer_free (gpointer key, gpointer value, gpointer user_data)
+{
+	g_free (key);
+}
+
+
 /**
  * g_mime_header_destroy:
  * @header: header object
@@ -105,32 +129,13 @@ g_mime_header_destroy (GMimeHeader *header)
 		}
 		
 		g_hash_table_destroy (header->hash);
+		g_hash_table_foreach (header->write_hash, writer_free, NULL);
+		g_hash_table_destroy (header->write_hash);
 		g_free (header);
 	}
 }
 
 
-/**
- * g_mime_header_foreach:
- * @header: header object
- * @func: function to be called for each header.
- * @data: User data to be passed to the func.
- *
- * Calls @func for each header name/value pair.
- */
-void
-g_mime_header_foreach (const GMimeHeader *header, GMimeHeaderFunc func, gpointer data)
-{
-	const struct raw_header *h;
-	
-	g_return_if_fail (header != NULL);
-	g_return_if_fail (header->hash != NULL);
-	
-	for (h = header->headers; h != NULL; h = h->next)
-		(*func) (h->name, h->value, data);
-}
-
-
 /**
  * g_mime_header_set:
  * @header: header object
@@ -152,18 +157,18 @@ g_mime_header_set (GMimeHeader *header, const char *name, const char *value)
 	
 	if ((h = g_hash_table_lookup (header->hash, name))) {
 		g_free (h->value);
-		if (value)
-			h->value = g_mime_utils_8bit_header_encode (value);
-		else
-			h->value = NULL;
+		h->value = g_strdup (value);
 	} else {
 		n = g_new (struct raw_header, 1);
 		n->next = NULL;
 		n->name = g_strdup (name);
-		n->value = value ? g_mime_utils_8bit_header_encode (value) : NULL;
+		n->value = g_strdup (value);
+	
+		h = header->headers;
+		while (h && h->next)
+			h = h->next;
 		
-		for (h = header->headers; h && h->next; h = h->next);
-		if (h)
+		if (h != NULL)
 			h->next = n;
 		else
 			header->headers = n;
@@ -194,7 +199,7 @@ g_mime_header_add (GMimeHeader *header, const char *name, const char *value)
 	n = g_new (struct raw_header, 1);
 	n->next = NULL;
 	n->name = g_strdup (name);
-	n->value = value ? g_mime_utils_8bit_header_encode (value) : NULL;
+	n->value = g_strdup (value);
 	
 	for (h = header->headers; h && h->next; h = h->next);
 	
@@ -267,33 +272,46 @@ g_mime_header_remove (GMimeHeader *header, const char *name)
 }
 
 
+
+
 /**
  * g_mime_header_write_to_stream:
  * @header: header object
  * @stream: output stream
  *
  * Write the headers to a stream.
+ *
+ * Returns the number of bytes written or -1 on fail.
  **/
-void
+gssize
 g_mime_header_write_to_stream (const GMimeHeader *header, GMimeStream *stream)
 {
+	GMimeHeaderWriteFunc header_write;
+	gssize nwritten, total = 0;
 	struct raw_header *h;
 	
-	g_return_if_fail (header != NULL);
-	g_return_if_fail (stream != NULL);
+	g_return_val_if_fail (header != NULL, -1);
+	g_return_val_if_fail (stream != NULL, -1);
 	
 	h = header->headers;
 	while (h) {
-		char *val;
-		
 		if (h->value) {
-			val = g_mime_utils_header_printf ("%s: %s\n", h->name, h->value);
-			g_mime_stream_write_string (stream, val);
-			g_free (val);
+			header_write = g_hash_table_lookup (header->write_hash, h->name);
+			if (header_write)
+				nwritten = (*header_write) (stream, h->name, h->value);
+			else
+				nwritten = (header->default_write_func)(stream, h->name, h->value);
+			
+			if (nwritten == -1)
+				return -1;
+			
+			total += nwritten;
 		}
 		
 		h = h->next;
 	}
+	
+	return total;
 }
 
 
@@ -326,3 +344,61 @@ g_mime_header_to_string (const GMimeHeader *header)
 	
 	return str;
 }
+
+
+/**
+ * g_mime_header_foreach:
+ * @header: header object
+ * @func: function to be called for each header.
+ * @user_data: User data to be passed to the func.
+ *
+ * Calls @func for each header name/value pair.
+ */
+void
+g_mime_header_foreach (const GMimeHeader *header, GMimeHeaderForeachFunc func, gpointer user_data)
+{
+	const struct raw_header *h;
+	
+	g_return_if_fail (header != NULL);
+	g_return_if_fail (header->hash != NULL);
+	g_return_if_fail (func != NULL);
+	
+	for (h = header->headers; h != NULL; h = h->next)
+		(*func) (h->name, h->value, user_data);
+}
+
+
+/**
+ * g_mime_header_set_write_func:
+ * @header: header object
+ * @name: header name
+ * @func: writer function
+ *
+ * Changes the function used to write @name headers to @func. This is
+ * useful if you want to change the default header folding style for a
+ * particular header.
+ **/
+void
+g_mime_header_set_write_func (GMimeHeader *header, const char *name, GMimeHeaderWriteFunc func)
+{
+	gpointer okey, oval;
+	
+	g_return_if_fail (header != NULL);
+	g_return_if_fail (name != NULL);
+	
+	if (g_hash_table_lookup (header->write_hash, name)) {
+		g_hash_table_lookup_extended (header->write_hash, name, &okey, &oval);
+		g_hash_table_remove (header->write_hash, name);
+		g_free (okey);
+	}
+	
+	g_hash_table_insert (header->write_hash, g_strdup (name), func);
+}
+
+void
+g_mime_header_set_default_write_func (GMimeHeader * header, GMimeHeaderWriteFunc func)
+{
+	g_return_if_fail (header != NULL);
+
+	header->default_write_func = func;
+}
