@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,22 +33,22 @@
 #include <pan/filters/filter-phrase.h>
 
 #include <pan/article-toolbar.h>
-#include <pan/filter-current-ui.h>
+#include <pan/filter-mediator.h>
 #include <pan/globals.h>
 #include <pan/util.h>
 
-#include <pan/xpm/pan-pixbufs.h>
-
 static void fire_user_changed_filter (void);
 
-static GtkWidget * _filter_phrase_om = NULL;
 static GtkWidget * _filter_phrase_entry = NULL;
 
-static char * _filter_phrase = NULL;
-static gulong _filter_bits =  ~STATE_FILTER_IGNORED;
-static FilterPhraseState _filter_phrase_state = FILTER_PHRASE_SUBJECT;
-static FilterShow _filter_show = FILTER_SHOW_MATCHES_AND_REFERENCES;
-static gboolean _filter_negate = FALSE;
+/*
+ * FIXME: these should be made threadsafe is we're to allow 
+ * multiple group switches in parallel (see switch_mutex in articlelist)
+ */
+
+static PString _filter_phrase;
+
+static PhraseKeyType _phrase_key_type = PHRASE_KEY_SUBJECT;
 static Filter* _filter = NULL;
 static Filter* _raw_filter = NULL;
 
@@ -62,35 +60,27 @@ extern GtkTooltips * ttips;
 ***
 **/
 
-static char*
-get_filter_phrase (void)
-{
-	return g_strdup (_filter_phrase);
-}
-
 static gboolean
 update_filter_phrase (void)
 {
-	char * tmp = g_strdup (gtk_entry_get_text (GTK_ENTRY(_filter_phrase_entry)));
-	gboolean changed = FALSE;
-	g_strstrip (tmp);
+	gboolean changed;
+	PString new_phrase;
 
-	if (!pan_strcmp (tmp, _filter_phrase))
-		g_free (tmp);
-	else {
-		changed = TRUE;
-		replace_gstr (&_filter_phrase, tmp);
-		if (!is_nonempty_string (_filter_phrase))
-			replace_gstr (&_filter_phrase, NULL);
-	}
+	new_phrase = pstring_shallow (gtk_entry_get_text (GTK_ENTRY (_filter_phrase_entry)), -1);
+	new_phrase = pstring_strstrip_shallow (&new_phrase);
+
+	changed = !pstring_equal (&_filter_phrase, &new_phrase);
+
+	if (changed)
+		pstring_copy (&_filter_phrase, &new_phrase);
 
 	return changed;
 }
 
-static FilterPhraseState
-get_filter_phrase_state (void)
+static PhraseKeyType
+get_phrase_key_type (void)
 {
-	return _filter_phrase_state;
+	return _phrase_key_type;
 }
 
 
@@ -98,28 +88,33 @@ static Filter*
 merge_filter_with_phrase (Filter * filter)
 {
 	Filter * retval = NULL;
-	char * phrase_str = get_filter_phrase ();
 
-	if (is_nonempty_string (phrase_str))
+	if (!_filter_phrase.len)
+		retval = filter;
+	else /* add the phrase filter to `filter' */
 	{
+		char * key = g_strndup (_filter_phrase.str, _filter_phrase.len);
 		Filter * phrase_filter = filter_phrase_new ();
-		Filter * children[2] = {filter, phrase_filter};
+		Filter * children[2];
+
+		children[0] = filter;
+		children[1] = phrase_filter;
 
 		filter_phrase_set (FILTER_PHRASE(phrase_filter),
-		                   get_filter_phrase_state(),
-		                   phrase_str,
+		                   PHRASE_MATCH_CONTAINS,
+		                   get_phrase_key_type(),
+		                   key,
 		                   FALSE);
 
 		retval = filter_aggregate_new ();
 		filter_aggregate_add (FILTER_AGGREGATE(retval), children, G_N_ELEMENTS(children));
 
+		/* cleanup */
 		pan_object_unref (PAN_OBJECT(children[0]));
 		pan_object_unref (PAN_OBJECT(children[1]));
+		g_free (key);
 	}
-	else retval = filter;
 
-	/* cleanup */
-	g_free (phrase_str);
 	return retval;
 }
 
@@ -130,13 +125,16 @@ merge_filter_with_phrase (Filter * filter)
 ***/
 
 static void
-rebuild_filters ()
+rebuild_filters (void)
 {
+	gulong bits;
+	FilterShow show;
 	Filter * old;
 
 	/* update the raw value */
 	old = _raw_filter;
-	_raw_filter = filter_current_dialog_get_filter (_filter_bits, _filter_negate);
+	filter_mediator_get_bits (&bits, &show);
+	_raw_filter = filter_mediator_get_filter (bits, FALSE);
 	pan_object_unref (PAN_OBJECT(old));
 
 	/* update the main filter */
@@ -145,14 +143,17 @@ rebuild_filters ()
 	pan_object_unref (PAN_OBJECT(old));
 }
 
+static void 
+filter_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data);
+
 void
 article_toolbar_set_group (const Group * group)
 {
 	g_return_if_fail (group_is_valid (group));
 
-	_filter_bits = group->filter_bits;
-	_filter_show = group->filter_show;
-	_filter_negate = FALSE; /* FIXME */
+	pan_callback_remove (filter_mediator_get_change_callback(), filter_changed_cb, NULL);
+	filter_mediator_set_bits (group->filter_bits, group->filter_show);
+	pan_callback_add (filter_mediator_get_change_callback(), filter_changed_cb, NULL);
 
 	rebuild_filters ();
 }
@@ -160,12 +161,14 @@ article_toolbar_set_group (const Group * group)
 void
 article_toolbar_set_group_filter (Group * group)
 {
+	gulong bits;
+	FilterShow show;
 	g_return_if_fail (group_is_valid (group));
 
-	group->filter_bits = _filter_bits;
-	group->filter_show = _filter_show;
-	group->filter_name = NULL; /* FIXME */
-	/* FIXME: add filter_negate */
+	filter_mediator_get_bits (&bits, &show);
+	group->filter_bits = bits;
+	group->filter_show = show;
+	group_set_dirty (group);
 }
 
 void
@@ -176,7 +179,16 @@ article_toolbar_get_filter (Filter     ** newme_filter,
 	g_return_if_fail (show_setme!=NULL);
 
 	*newme_filter = filter_dup (_filter);
-	*show_setme = _filter_show;
+	filter_mediator_get_bits (NULL,  show_setme);
+}
+
+static gboolean
+grab_focus_idle (gpointer w)
+{
+	pan_lock ();
+	gtk_widget_grab_focus (GTK_WIDGET(w));
+	pan_unlock ();
+	return FALSE;
 }
 
 static gboolean
@@ -186,18 +198,19 @@ article_phrase_activate_cb (void)
 	{
 		rebuild_filters ();
 		fire_user_changed_filter ();
+		gui_queue_add (grab_focus_idle, Pan.article_ctree);
 	}
 	return FALSE;
 }
 
 static void
-set_filter_mode_cb (GtkMenuItem * item, gpointer data)
+set_key_type_cb (GtkMenuItem * item, gpointer data)
 {
-	FilterPhraseState tmp = GPOINTER_TO_INT(data);
+	PhraseKeyType key_type = GPOINTER_TO_INT(data);
 	
-	if (tmp != _filter_phrase_state)
+	if (key_type != _phrase_key_type)
 	{
-		_filter_phrase_state = tmp;
+		_phrase_key_type = key_type;
 		rebuild_filters ();
 		fire_user_changed_filter ();
 	}
@@ -209,48 +222,18 @@ set_filter_mode_cb (GtkMenuItem * item, gpointer data)
 **** 
 ***/
 
-static void
-filter_current_response_cb (GtkDialog * dialog, int response, gpointer user_data)
-{
-	if (response == GTK_RESPONSE_APPLY || response == GTK_RESPONSE_OK)
-	{
-		filter_current_dialog_get_bits (GTK_WIDGET(dialog),
-		                                &_filter_bits,
-		                                &_filter_show,
-		                                &_filter_negate);
-		rebuild_filters ();
-		fire_user_changed_filter ();
-	}
 
-	if (response != GTK_RESPONSE_APPLY)
-		gtk_widget_destroy (GTK_WIDGET(dialog));
+static int
+filter_changed_idle (gpointer unused)
+{
+	rebuild_filters ();
+	fire_user_changed_filter ();
+	return 0;
 }
-
-static void
-widget_destroyed_cb (GtkObject * o, gpointer data)
+static void 
+filter_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
-	        *((GtkWidget**)data) = NULL;
-}
-
-static GtkWidget * filter_current_dialog = NULL;
-
-void
-article_toolbar_edit_filter (void)
-{
-	if (filter_current_dialog == NULL)
-	{
-		filter_current_dialog = filter_current_dialog_new (GTK_WINDOW(Pan.window),
-			_filter_bits,
-			_filter_show,
-			_filter_negate);
-
-		g_signal_connect (GTK_OBJECT(filter_current_dialog), "destroy",
-		                  G_CALLBACK(widget_destroyed_cb), &filter_current_dialog);
-		g_signal_connect (GTK_OBJECT(filter_current_dialog), "response",
-		                  G_CALLBACK(filter_current_response_cb), NULL);
-	}
-
-	gtk_widget_show_all (filter_current_dialog);
+	gui_queue_add (filter_changed_idle, NULL);
 }
 
 extern GtkAccelGroup * _main_accel_group;
@@ -276,57 +259,41 @@ entry_focus_out_cb (GtkWidget * w, GdkEventKey * event, gpointer unused) {
 
 typedef struct
 {
-	const gchar * name;
-	FilterPhraseState state;
+	const char * name;
+	PhraseKeyType state;
 }
 ArticleToolbarOptionMenuStruct;
 
 GtkWidget*
 article_toolbar_new (void)
 {
+	gulong bits;
+	FilterShow show;
 	GtkWidget * w;
 	GtkWidget * toolbar;
-	char buf[512];
 
 	/**
 	***  One-time initialization of local fields
 	**/
 
-	_filter_bits = ~(gulong)STATE_FILTER_IGNORED;
-	g_snprintf (buf, sizeof(buf), "/Display/filter_bits=%lu", _filter_bits);
-	_filter_bits = pan_config_get_ulong (buf);
+	bits = ~(gulong)(STATE_FILTER_SCORE_IGNORED);
 
-	_filter_show = FILTER_SHOW_MATCHES_AND_REFERENCES;
-	g_snprintf (buf, sizeof(buf), "/Display/filter_show=%lu", (gulong)_filter_show);
-	_filter_show = pan_config_get_ulong (buf);
+	show = FILTER_SHOW_MATCHES;
 
-	_filter_negate = pan_config_get_bool ("/Display/filter_negate=false");
+	filter_mediator_set_bits (bits, show);
 
-	_raw_filter = filter_current_dialog_get_filter (_filter_bits, _filter_negate);
+	_raw_filter = filter_mediator_get_filter (bits, FALSE);
 
 	_filter = filter_dup (_raw_filter);
 
+	pan_callback_add (filter_mediator_get_change_callback(), filter_changed_cb, NULL);
+
 	/**
 	***  Create the GUI
 	**/
 
 	toolbar = gtk_hbox_new (FALSE, GUI_PAD);
 
-	/* filter button */
-	if (1) {
-		GdkPixbuf * p = gdk_pixbuf_new_from_inline (-1, icon_filter, FALSE, NULL);
-		GtkWidget * i = gtk_image_new_from_pixbuf (p);
-		g_object_unref (G_OBJECT(p));
-		w = gtk_button_new ();
-		g_signal_connect (w, "clicked", G_CALLBACK(article_toolbar_edit_filter), NULL);
-		gtk_container_add (GTK_CONTAINER(w), i);
-		gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
-		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Header Pane Filter..."), "");
-	}
-
-	w = gtk_label_new (_("Find:"));
-	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
-
 	/* filter by phrase */
 	if (1) {
 		GtkWidget * option_menu = gtk_option_menu_new ();
@@ -334,9 +301,9 @@ article_toolbar_new (void)
 		int index = 0;
 		int i;
 		ArticleToolbarOptionMenuStruct foo[] = {
-			{NULL, FILTER_PHRASE_SUBJECT},
-			{NULL, FILTER_PHRASE_AUTHOR},
-			{NULL, FILTER_PHRASE_MESSAGE_ID}
+			{NULL, PHRASE_KEY_SUBJECT},
+			{NULL, PHRASE_KEY_AUTHOR},
+			{NULL, PHRASE_KEY_MESSAGE_ID}
 		};
 		const int row_qty = G_N_ELEMENTS (foo);
 		foo[0].name = _("Subject");
@@ -345,7 +312,7 @@ article_toolbar_new (void)
 		for (i=0; i<row_qty; ++i) {
 			GtkWidget * item = gtk_menu_item_new_with_label (foo[i].name);
 			gtk_signal_connect (GTK_OBJECT(item), "activate",
-			                    GTK_SIGNAL_FUNC(set_filter_mode_cb), GINT_TO_POINTER(foo[i].state));
+			                    GTK_SIGNAL_FUNC(set_key_type_cb), GINT_TO_POINTER(foo[i].state));
 			gtk_menu_append (GTK_MENU(menu), item);
 			gtk_widget_show (item);
 		}
@@ -355,14 +322,20 @@ article_toolbar_new (void)
 		w = option_menu;
 	}
 	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
-	_filter_phrase_om = w;
-	_filter_phrase_entry = w = gtk_entry_new ();
+
+	w = gtk_label_new_with_mnemonic (_("Fin_d:"));
+	gtk_box_pack_start (GTK_BOX(toolbar), w, FALSE, FALSE, 0);
+
+	_filter_phrase = PSTRING_INIT;
+	_filter_phrase_entry = gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL(w), _filter_phrase_entry);
+	w = _filter_phrase_entry;
 	g_signal_connect (w, "focus_in_event", G_CALLBACK(entry_focus_in_cb), NULL);
 	g_signal_connect (w, "focus_out_event", G_CALLBACK(entry_focus_out_cb), NULL);
-        g_signal_connect (w, "focus_out_event", G_CALLBACK(article_phrase_activate_cb), NULL);
+	g_signal_connect (w, "focus_out_event", G_CALLBACK(article_phrase_activate_cb), NULL);
 	g_signal_connect (w, "activate", G_CALLBACK(article_phrase_activate_cb), NULL);
 	gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w,
-	        _("Type in a search string and press ENTER.\nWildcards are allowed."), "");
+	        _("Type in a search string and press ENTER.\nWildcards are allowed."), NULL);
 	gtk_box_pack_start (GTK_BOX(toolbar), w, TRUE, TRUE, 0);
 
 	/* return the toolbar widget */
@@ -385,10 +358,12 @@ article_toolbar_get_user_changed_filter_callback (void)
 static void
 fire_user_changed_filter (void)
 {
+	FilterShow show;
 	ArticleToolbarCallbackStruct cbs;
 	debug_enter ("fire_filter_changed");
+	filter_mediator_get_bits (NULL, &show);
 	cbs.filter = _filter;
-	cbs.filter_show = _filter_show;
+	cbs.filter_show = show;
 	pan_callback_call (article_toolbar_get_user_changed_filter_callback (), &cbs, NULL);
 
 	debug_exit ("fire_filter_changed");
