@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,18 +15,18 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
 
-#include <glib.h>
+#include <glib/gmem.h>
 
 #include <ctype.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <pan/base/acache.h>
 #include <pan/base/article.h>
 #include <pan/base/article-thread.h>
 #include <pan/base/base-prefs.h>
@@ -53,55 +51,88 @@
  */
 typedef struct
 {
-	gchar * subject;
 	gboolean is_reply;
+	int subject_len;
+	char * subject;
 	Article * a;
 }
 Norm;
 
 /**
- * Normalizing a subject header involves tearing out the multipart
- * substrings ("(21/42)" or "[12|213]") and converting it all to
- * one case so that we can use strcmp instead of g_strcasecmp.
+ * Normalizing a subject header involves:
  *
- * When we're threading articles, it's a big speedup to normalize the
- * subjects at the outset instead of normalizing them in each comparison.
+ * 1. tearing out the numerator from multipart indicators
+ *    (i.e., remove "21" from (21/42))
+ *    for threading.
+ * 2. convert it to lowercase so that, when sorting, we can
+ *    have case-insensitive sorting without having to use
+ *    a slower case-insensitive compare function.
+ * 3. strip out all the leading noise that breaks sorting.
+ *
+ * When we're threading articles, it's much faster to
+ * normalize the * subjects at the outset instead of
+ * normalizing them for each comparison.
  */
-static size_t
-normalize_subject (char * buf, const Article * a)
+size_t
+normalize_subject (char            * buf,
+                   const Article   * a,
+                   StripFlags        strip)
 {
 	static gboolean _inited = FALSE;
 	static gboolean _keep_chars[UCHAR_MAX+1];
-	register const guchar * in = (guchar*) skip_reply_leader (a->subject);
+	register const guchar * in = (guchar*) skip_reply_leader (a->subject.str);
 	register char * out = buf;
 	const gboolean multipart = a->parts != 0;
+	const gboolean strip_case = strip & STRIP_CASE;
+	const gboolean strip_numerator = strip & STRIP_MULTIPART_NUMERATOR;
+	const gboolean strip_multipart = strip & STRIP_MULTIPART;
 
 	/* populate the _keep_chars array */
 	if (!_inited) {
 		int i;
 		for (i=0; i<=UCHAR_MAX; ++i) {
-			guchar uch = (guchar)i;
+			const guchar uch = (guchar)i;
 			_keep_chars[i] = isalnum(uch) || isdigit(uch) || isspace(uch);
 		}
 		_inited = TRUE;
 	}
 
 	/* skip the leading noise */
-	while (*in && !isalnum(*in))
+	while (*in && isspace(*in))
 		++in;
 
 	while (*in)
 	{
-		/* strip multipart information */
-		if (multipart && (*in=='('||*in=='[') && isdigit(in[1])) {
-			while (*in && *in!=']' && *in!=')')
-				++in;
+		/* strip numerator out of multipart information */
+		if ((strip_multipart||strip_numerator) && multipart && (*in=='('||*in=='[') && isdigit(in[1]))
+		{
+			const guchar * start = ++in;
+
+			if (strip_multipart || strip_numerator)
+			{
+				while (*in && isdigit(*in))
+					++in;
+
+				if (*in!='/' && *in!='|') /* oops, not multipart information */
+					in = start;
+
+				else if (strip_multipart)
+				{
+					for (++in; *in && *in!=']' && *in!=')'; ++in)
+					{
+						if (isalpha(*in)) { /* oops, not multipart information */
+							in = ++start;
+							break;
+						}
+					}
+				}
+			}
 			continue;
 		}
 
-		/* strip out junk that breaks sorting  */
+		/* strip out junk that breaks sorting */
 		if (_keep_chars[*in])
-			*out++ = tolower(*in);
+			*out++ = (char) (strip_case ? tolower(*in) : *in);
 
 		++in;
 	}
@@ -118,12 +149,13 @@ normalize_subject (char * buf, const Article * a)
 static void
 normalize_articles (Article    ** articles,
                     gint          qty,
+                    StripFlags    strip_flags,
                     Norm       ** alloc_and_setme_norm,
                     gchar      ** alloc_and_setme_str)
 {
 	gint i;
 	glong str_buf_idx;
-	glong str_len;
+	glong len;
 	gchar * str_buf;
 	Norm * norm_buf;
 
@@ -138,19 +170,20 @@ normalize_articles (Article    ** articles,
 	*alloc_and_setme_norm = norm_buf = g_new (Norm, qty);
 
 	/* alloc a buf for the subject */
-	str_len = 0;
+	len = 0;
 	for (i=0; i<qty; ++i)
-		str_len += strlen (articles[i]->subject) + 2;
-	*alloc_and_setme_str = str_buf = g_new (char, str_len);
+		len += articles[i]->subject.len + 2;
+	*alloc_and_setme_str = str_buf = g_new (char, len);
 	
 	/* normalize the articles */
 	str_buf_idx = 0;
 	for (i=0; i<qty; ++i) {
 		Article * a = articles[i];
 		norm_buf[i].a = a;
-		norm_buf[i].is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
+		norm_buf[i].is_reply = skip_reply_leader (a->subject.str) != a->subject.str ? 1 : 0;
 		norm_buf[i].subject = str_buf + str_buf_idx;
-		str_buf_idx += normalize_subject (norm_buf[i].subject, a) + 1;
+		norm_buf[i].subject_len = normalize_subject (norm_buf[i].subject, a, strip_flags);
+		str_buf_idx += norm_buf[i].subject_len + 1;
 	}
 }
 
@@ -165,19 +198,19 @@ compare_pA_to_pA_by_part (const void * va, const void * vb)
 	gboolean        b_is_reply;
 
 	/* already normalized, so check multipart first */
-	if (a->parts == b->parts)
+	if (a->parts!=0 && (a->parts == b->parts))
 		return a->part - b->part;
 
 	/* order by subject */
-	if ((value = strcmp (a->subject, b->subject)))
+	if ((value = pstring_compare (&a->subject, &b->subject)))
 		return value;
 
 	/* the rest is probably unnecessary. just carry-over from
 	 * compare_pA_to_pA_by_subject */
 
 	/* if one but not both is a reply, the reply goes second */
-	a_is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
-	b_is_reply = skip_reply_leader (b->subject) != b->subject ? 1 : 0;
+	a_is_reply = skip_reply_leader (a->subject.str) != a->subject.str ? 1 : 0;
+	b_is_reply = skip_reply_leader (b->subject.str) != b->subject.str ? 1 : 0;
 	if (a_is_reply != b_is_reply)
 		return a_is_reply ? 1 : -1;
 
@@ -195,13 +228,13 @@ static int
 compare_pN_to_pN_by_subject (const void * va, const void * vb)
 {
 	register int value;
-	const register Norm * a = (const Norm *)va;
-	const register Norm * b = (const Norm *)vb;
+	register const Norm * a = (const Norm *)va;
+	register const Norm * b = (const Norm *)vb;
 
 	/* subject is the primary key, of course... */
 	if ((value = *a->subject - *b->subject))
 		return value;
-	if ((value = strcmp (a->subject, b->subject)))
+	if ((value = pan_strcmp_len (a->subject, a->subject_len, b->subject, b->subject_len)))
 		return value;
 
 	/* if one but not both is a reply, the reply goes second */
@@ -222,39 +255,59 @@ compare_pN_to_pN_by_subject (const void * va, const void * vb)
 	return value;
 }
 
+static int
+compare_ppA_to_ppA_by_score (const void* va, const void* vb)
+{
+        register const Article * a = *(const Article**)va;
+        register const Article * b = *(const Article**)vb;
+	return a->score - b->score;
+}
+
+static gulong
+article_get_combined_linecount (const Article * a)
+{
+	gulong retval = 0;
+
+	g_return_val_if_fail (article_is_valid(a), FALSE);
+
+	retval = a->linecount;
+	if (a->part!=0 && a->parts!=0 && a->threads!=NULL) {
+		GSList * l;
+		int part = a->part + 1;
+		for (l=a->threads; l; l=l->next) {
+			Article * child = ARTICLE(l->data);
+			if (child->part == part) {
+				retval += child->linecount;
+				++part;
+			}
+		}
+	}
+
+	return retval;
+}
 
 static int
 compare_ppA_to_ppA_by_linecount (const void* va, const void* vb)
 {
-        const register Article * a = *(const Article**)va;
-        const register Article * b = *(const Article**)vb;
+        register const Article * a = *(const Article**)va;
+        register const Article * b = *(const Article**)vb;
 	return article_get_combined_linecount(a) - article_get_combined_linecount(b);
 }
 
 static int
 compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 {
-	const register Article * a = *(const Article **)va;
-	const register Article * b = *(const Article **)vb;
-	gint ia, ib;
+	register const Article * a = *(const Article **)va;
+	register const Article * b = *(const Article **)vb;
+	int ia, ib;
 
-	ia = article_flag_on (a, STATE_SAVE_QUEUED) ? 1 : 0;
-	ib = article_flag_on (b, STATE_SAVE_QUEUED) ? 1 : 0;
+	ia = article_get_decode_state (a);
+	ib = article_get_decode_state (b);
 	if (ia != ib)
 		return ib - ia;
 
-	ia = article_flag_on (a, STATE_DOWNLOAD_FLAGGED) ? 1 : 0;
-	ib = article_flag_on (b, STATE_DOWNLOAD_FLAGGED) ? 1 : 0;
-	if (ia != ib)
-		return ib - ia;
-
-	ia = article_flag_on (a, STATE_DECODED) ? 1 : 0;
-	ib = article_flag_on (b, STATE_DECODED) ? 1 : 0;
-	if (ia != ib)
-		return ib - ia;
-
-	ia = article_has_body (a) ? 1 : 0;
-	ib = article_has_body (b) ? 1 : 0;
+	ia = acache_has_message (group_get_acache_key(a->group), &a->message_id) ? 1 : 0;
+	ib = acache_has_message (group_get_acache_key(b->group), &b->message_id) ? 1 : 0;
 	if (ia != ib)
 		return ib - ia;
 
@@ -264,8 +317,8 @@ compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 static int
 compare_ppA_to_ppA_by_read (const void * va, const void * vb)
 {
-	const register Article * a = *(const Article **)va;
-	const register Article * b = *(const Article **)vb;
+	register const Article * a = *(const Article **)va;
+	register const Article * b = *(const Article **)vb;
 	const gboolean a_is_read = article_is_read (a);
 	const gboolean b_is_read = article_is_read (b);
 	const gboolean a_is_new = article_is_new (a);
@@ -287,13 +340,8 @@ compare_ppA_to_ppA_by_read (const void * va, const void * vb)
 	if (ia != ib)
 		return ib - ia;
 
-	ia = article_flag_on (a, STATE_MULTIPART_ALL) ? 1 : 0;
-	ib = article_flag_on (b, STATE_MULTIPART_ALL) ? 1 : 0;
-	if (ia != ib)
-		return ib - ia;
-
-	ia = article_flag_on (a, STATE_MULTIPART_SOME) ? 1 : 0;
-	ib = article_flag_on (b, STATE_MULTIPART_SOME) ? 1 : 0;
+	ia = article_get_multipart_state (a);
+	ib = article_get_multipart_state (b);
 	if (ia != ib)
 		return ib - ia;
 
@@ -318,16 +366,17 @@ compare_ppA_to_ppA_by_date (const void* va, const void* vb)
 }
 
 static int
-compare_ppA_to_ppA_by_message_id (const void* a, const void* b)
+compare_ppA_to_ppA_by_message_id (const void* va, const void* vb)
 {
-	const gchar * msg_id_a = (*(const Article**)a)->message_id;
-	const gchar * msg_id_b = (*(const Article**)b)->message_id;
-	return strcmp (msg_id_a, msg_id_b);
+	register const Article * a = *(const Article **)va;
+	register const Article * b = *(const Article **)vb;
+	return pstring_compare (&a->message_id, &b->message_id);
 }
 
 typedef struct
 {
 	char name[128];
+	int name_len;
 	Article * article;
 }
 ArticleStruct;
@@ -337,7 +386,7 @@ compare_pAS_to_pAS_by_data (const void * va, const void * vb)
 {
 	const ArticleStruct * a = (const ArticleStruct*)va;
 	const ArticleStruct * b = (const ArticleStruct*)vb;
-	return strcmp (a->name, b->name);
+	return pan_strcmp_len (a->name, a->name_len, b->name, b->name_len);
 }
 
 void
@@ -348,6 +397,9 @@ sort_articles (Article      ** buf,
 {
 	g_return_if_fail (articles_are_valid ((const Article **)buf, article_qty));
 
+	if (!article_qty)
+		return;
+
 	switch (sort_type)
 	{
 		case ARTICLE_SORT_AUTHOR:
@@ -357,7 +409,7 @@ sort_articles (Article      ** buf,
 			for (i=0; i<article_qty; ++i)
 			{
 				as[i].article = buf[i];
-				article_get_short_author_str (buf[i], as[i].name, sizeof(as[i].name));
+				as[i].name_len = article_get_short_author_str (buf[i], as[i].name, sizeof(as[i].name));
 				g_strdown (as[i].name);
 			}
 			msort (as,
@@ -394,13 +446,18 @@ sort_articles (Article      ** buf,
 			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_read);
 			break;
 		}
+		case ARTICLE_SORT_SCORE:
+		{
+			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_score);
+			break;
+		}
 		case ARTICLE_SORT_SUBJECT:
 		default:
 		{
 			gint i;
 			Norm * norm_buf = NULL;
 			char * str_buf = NULL;
-			normalize_articles (buf, article_qty, &norm_buf, &str_buf);
+			normalize_articles (buf, article_qty, STRIP_CASE, &norm_buf, &str_buf);
 			msort (norm_buf, article_qty, sizeof(Norm), compare_pN_to_pN_by_subject);
 			for (i=0; i<article_qty; ++i)
 				buf[i] = ARTICLE(norm_buf[i].a);
@@ -439,40 +496,57 @@ is_child_of (const Article * child,
 	}
 }
 
-static guint
-article_get_part_state (Article * a)
+static int
+article_root_get_part_state (const Article * a)
 {
-	guint retval;
+	int retval;
 
+	/* sanity clause */
+	g_return_val_if_fail (a!=NULL, 0u);
+	g_return_val_if_fail (a->parent==NULL, 0u);
+
+	/* not a multipart */
 	if (a->parts<1)
-		retval = 0;
+		retval = MULTIPART_STATE_NONE;
+
+	/* someone's posted a "00/124" nfo message */
+	else if (a->part==0)
+		retval = MULTIPART_STATE_NONE;
+
+	/* incomplete multipart */
 	else if (a->part>1)
-		retval = STATE_MULTIPART_SOME;
+		retval = MULTIPART_STATE_SOME;
+
+	/* someone's posted a followup to a multipart */
+	else if (a->linecount<250 && !g_strncasecmp(a->subject.str,"Re:", 3))
+		retval = MULTIPART_STATE_NONE;
+
+	/* a multipart */
 	else {
 		GSList * l;
 		gint part = a->part + 1;
 		for (l=a->threads; l!=NULL; l=l->next)
 			if (ARTICLE(l->data)->parts==a->parts && ARTICLE(l->data)->part==part)
 				++part;
-		retval = part==a->parts+1 ? STATE_MULTIPART_ALL : STATE_MULTIPART_SOME;
+		retval = part==a->parts+1 ? MULTIPART_STATE_ALL : MULTIPART_STATE_SOME;
 	}
 
 	return retval;
 }
 
 static void
-set_children_part_state (Article * a, guint state)
+set_children_part_state (Article * a, const int state)
 {
 	gint part;
 	GSList * l;
 
-	a->state |= state;
+	a->multipart_state = state;
 
 	part = a->part + 1;
 	for (l=a->threads; l!=NULL; l=l->next) {
 		Article * child = ARTICLE(l->data);
 		if (child->parts==a->parts && child->part==part) {
-			child->state |= state;
+			child->multipart_state = state;
 			++part;
 		}
 	}
@@ -482,23 +556,22 @@ set_children_part_state (Article * a, guint state)
  * Thread the articles specified in list
  */
 void
-thread_articles (GPtrArray    * articles)
+thread_articles (Article   ** articles,
+                 guint        article_qty)
 {
 	guint i;
-	guint qty = articles->len;
 	Article search_a;
 	gchar * norm_str_buf;
 	Norm * norm;
 	Norm * sorted_norm;
 	GArray * buf = NULL;
-	GString * tmp_references;
-	GHashTable * ref_hash;
+	GHashTable * mid_to_article;
 
 	/* sanity clause */
 	g_return_if_fail (articles!=NULL);
-	g_return_if_fail (articles_are_valid ((const Article**)articles->pdata, articles->len));
+	g_return_if_fail (articles_are_valid ((const Article**)articles, article_qty));
 
-	if (qty<1 || !articles)
+	if (article_qty<1 || !articles)
 		return;
 
 	if (break_thread_when_subject_changes)
@@ -506,57 +579,57 @@ thread_articles (GPtrArray    * articles)
 
 	/* make a plausiably-legal article */
 	search_a.number = 1;
-	search_a.subject = "dummy subject";
-	search_a.references = NULL;
+	search_a.subject = pstring_shallow ("dummy subject", -1);
+	search_a.references = PSTRING_INIT;
 
 	/* make a message-id-sorted array of the articles */
-	ref_hash = g_hash_table_new (g_str_hash, g_str_equal);
-	for (i=0; i<qty; ++i) {
-		Article * a = ARTICLE (g_ptr_array_index (articles, i));
-		g_hash_table_insert (ref_hash, (gpointer)a->message_id, a);
-	}
+	mid_to_article = g_hash_table_new (pstring_hash, pstring_equal);
+	for (i=0; i<article_qty; ++i)
+		g_hash_table_insert (mid_to_article, &articles[i]->message_id, articles[i]);
 
 	/* normalize the articles */
 	norm = NULL;
 	norm_str_buf = NULL;
-	normalize_articles ((Article**)articles->pdata, qty, &norm, &norm_str_buf);
+	normalize_articles (articles, article_qty, STRIP_MULTIPART_NUMERATOR, &norm, &norm_str_buf);
 
 	/* sort the normalized articles */
-	sorted_norm = g_memdup (norm, sizeof(Norm)*qty);
-	qsort (sorted_norm, qty, sizeof(Norm), compare_pN_to_pN_by_subject);
+	sorted_norm = g_memdup (norm, sizeof(Norm)*article_qty);
+	qsort (sorted_norm, article_qty, sizeof(Norm), compare_pN_to_pN_by_subject);
 
 	/* unthread the articles, just in case they were threaded before */
-	for (i=0; i!=qty; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+	for (i=0; i!=article_qty; ++i)
+	{
+		Article * a = articles[i];
 		a->parent = NULL;
-		a->unread_children = 0;
-		a->new_children = 0;
 		g_slist_free (a->threads);
 		a->threads = NULL;
 	}
 
 
 	/* thread the articles */
-       	tmp_references = g_string_new (NULL);
-	for (i=0; i!=qty; ++i)
+	for (i=0; i!=article_qty; ++i)
 	{
-		const gchar * references;
+		PString references;
 		Article * parent = NULL;
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+		Article * a = articles[i];
 		gint index = -1;
 
-		/* thread by reference
-		   (except for parts 2...n of multiparts, which need to be threaded by multipart) */
+		/* thread by reference.
+		   (except for parts 2...n of multiparts, which need to be threaded by multipart).
+		   We take the references header and work our way from the last message-id backwards
+		   to the first.  For each message-id in this series, the first one we find in our
+		   set of articles is flagged as the parent. */
 		references = a->references;
-		if (a->parts<2 && references!=NULL && *references=='<')
+		if (a->parts<2 && pstring_is_set(&a->references))
 		{
-			gchar * message_id;
-			g_string_assign (tmp_references, references);
-		       	message_id = strrchr (tmp_references->str, '<');
+			/* pull the last message-id from the references string. */
+			const char * cpch = pan_strrchr_len (references.str, references.len, '<');
+			PString message_id = pstring_substr_shallow (&references, cpch, NULL);
 
-			while (parent==NULL && is_nonempty_string(message_id))
+			while (parent==NULL && pstring_is_set (&message_id))
 			{
-				Article * match = (Article*) g_hash_table_lookup (ref_hash, message_id);
+				/* look for a match in our message-id hashtable */
+				Article * match = (Article*) g_hash_table_lookup (mid_to_article, &message_id);
 
 				/* if we found the ancestor & it's worthy, thread it */
 				if (match!=NULL && !is_child_of(match,a))
@@ -565,11 +638,10 @@ thread_articles (GPtrArray    * articles)
 
 					if (break_thread_when_subject_changes)
 					{
-						const char * new_subject = norm[i].subject;
-						const char * old_subject = article_get_subject (match);
-						g_array_set_size (buf, strlen(old_subject)+1);
-						normalize_subject (buf->data, match);
-						subject_changed = strcmp (buf->data, new_subject);
+						int buf_len;
+						g_array_set_size (buf, match->subject.len + 1);
+						buf_len = normalize_subject (buf->data, match, STRIP_MULTIPART_NUMERATOR);
+						subject_changed = pan_strcmp_len (buf->data, buf_len, norm[i].subject, norm[i].subject_len);
 					}
 
 					if (!subject_changed)
@@ -577,13 +649,13 @@ thread_articles (GPtrArray    * articles)
 				}
 
 				/* if we couldn't find the ancestor, march up the References string */
-				*message_id = '\0';
-				g_strchomp (tmp_references->str);
-				message_id = strrchr (tmp_references->str, '<');
+				references.len -= message_id.len;
+				references = pstring_strstrip_shallow (&references);
+				cpch = pan_strrchr_len (references.str, references.len, '<');
+				message_id = pstring_substr_shallow (&references, cpch, NULL);
 			}
 		}
 
-
 		/* thread by multipart */
 		if (!parent && a->parts>1 && a->part>1)
 		{
@@ -594,17 +666,17 @@ thread_articles (GPtrArray    * articles)
 
 			index = lower_bound (&n,
 			                     sorted_norm,
-			                     qty,
+			                     article_qty,
 			                     sizeof(Norm),
 			                     compare_pN_to_pN_by_subject,
 			                     NULL);
 
-			if (0<=index && index<qty)
+			if (0<=index && index<article_qty)
 			{
 				Norm * match = &sorted_norm[index];
 				if ((match->a != a)
 					&& (match->a->parts == a->parts)
-					&& (!strcmp(match->subject,n.subject))
+					&& (!pan_strcmp_len(match->subject,match->subject_len,n.subject,n.subject_len))
 					&& (!is_child_of(match->a,a)))
 				{
 					parent = match->a;
@@ -613,7 +685,7 @@ thread_articles (GPtrArray    * articles)
 		}
 
 		/* thread by subject */
-		if (!parent && skip_reply_leader(a->subject)!=a->subject)
+		if (!parent && skip_reply_leader(a->subject.str)!=a->subject.str)
 		{
 			Norm n = norm[i];
 			search_a.part = 0;
@@ -623,21 +695,21 @@ thread_articles (GPtrArray    * articles)
 			index = lower_bound (
 				&n,
 				sorted_norm,
-				qty,
+				article_qty,
 				sizeof(Norm),
 				compare_pN_to_pN_by_subject,
 				NULL);
 
-			if (0<=index && index<qty && !is_child_of(sorted_norm[index].a,a))
+			if (0<=index && index<article_qty && !is_child_of(sorted_norm[index].a,a))
 			{
 				Norm * match = &sorted_norm[index];
 
-				if (!strcmp(match->subject,n.subject))
+				if (!pan_strcmp_len (match->subject, match->subject_len, n.subject, n.subject_len))
 				{
 					/* 1 original, 1 reply */
 					parent = match->a;
 				}
-				else if (!strcmp(match->subject, a->subject) && (match->a->date<a->date))
+				else if (!pan_strcmp_len (match->subject, match->subject_len, a->subject.str, a->subject.len) && (match->a->date<a->date))
 				{
 					/* 2 replies, no top --  oldest on top */
 					parent = match->a;
@@ -656,41 +728,30 @@ thread_articles (GPtrArray    * articles)
 	}
 
 	/* right now all the children are normalized; point to articles */
-	for (i=0; i!=qty; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+	for (i=0; i!=article_qty; ++i) {
+		Article * a = articles[i];
 		a->threads = g_slist_sort (a->threads, compare_pA_to_pA_by_part);
 	}
 
-	/* calculate new/unread child counts */
-	for (i=0; i!=qty; ++i)
-	{
-		Article * a;
+	/**
+	***  Calculate multipart states
+	**/
 
-		a = ARTICLE(g_ptr_array_index(articles,i));
-		if (a->parent!=NULL && !article_is_read(a))
-			for (a=a->parent; a!=NULL; a=a->parent)
-				++a->unread_children;
-
-		a = ARTICLE(g_ptr_array_index(articles,i));
-		if (a->parent!=NULL && article_is_new(a))
-			for (a=a->parent; a!=NULL; a=a->parent)
-				++a->new_children;
+	for (i=0; i!=article_qty; ++i) {
+		Article * a = articles[i];
+		a->multipart_state = MULTIPART_STATE_NONE;
 	}
-
-	/* check multipart count */
-	for (i=0; i!=articles->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		a->state &= ~(STATE_MULTIPART_ALL&STATE_MULTIPART_SOME);
+	for (i=0; i!=article_qty; ++i) {
+		Article * a = articles[i];
 		if (a->parent == NULL)
-			set_children_part_state (a, article_get_part_state(a));
+			set_children_part_state (a, article_root_get_part_state(a));
 	}
 
 	/* cleanup */
-	g_hash_table_destroy (ref_hash);
+	g_hash_table_destroy (mid_to_article);
 	g_free (norm);
 	g_free (norm_str_buf);
 	g_free (sorted_norm);
-	g_string_free (tmp_references, TRUE);
 	if (buf != NULL)
 		g_array_free (buf, TRUE);
 }
