@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -31,10 +28,13 @@
 #include <gmime/gmime-utils.h>
 #include <gmime/internet-address.h>
 
+#include <pan/base/article.h>
+#include <pan/base/base-prefs.h>
 #include <pan/base/gnksa.h>
 #include <pan/base/message-check.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/text-massager.h>
 
 /***
 ****  PRIVATE UTILITIES
@@ -62,6 +62,93 @@ get_nntp_rcpts (const gchar *to)
 ****  OUTGOING MESSAGE CHECKS
 ***/
 
+static char*
+strip_attribution_and_signature (const char * body_in, GMimeMessage * message)
+{
+	const char * attribution;
+	char * sig_delimiter;
+	GString * body;
+
+	/* sanity clause */
+	g_return_val_if_fail (body_in!=NULL, NULL);
+	g_return_val_if_fail (message!=NULL, NULL);
+
+	/* initialize the gstring */
+	body = g_string_new (body_in);
+	body_in = NULL;
+
+	/* strip out the attribution */
+	attribution = g_mime_message_get_header (message, PAN_ATTRIBUTION);
+	if (is_nonempty_string (attribution))
+	{
+		const char * attribution_start_pos = strstr (body->str, attribution);
+		if (attribution_start_pos != NULL)
+		{
+			const int attribution_start_index = attribution_start_pos - body->str;
+			const int attribution_len = strlen(attribution) + 2; /* +2: trim out the following carriage returns */
+			g_string_erase (body, attribution_start_index, attribution_len);
+		}
+	}
+
+	/* strip out the signature */
+	sig_delimiter = NULL;
+	if (pan_find_signature_delimiter (body->str, &sig_delimiter) != SIG_NONE) {
+		g_string_truncate (body, sig_delimiter - body->str);
+		pan_g_string_strstrip (body);
+	}
+
+	return g_string_free (body, FALSE);
+}
+
+/**
+ * Check to see if the user is top-posting.
+ */
+static void
+check_topposting (const TextMassager   * tm,
+                  GPtrArray            * setme_errors,
+                  GoodnessLevel        * setme_goodness,
+                  const char           * body,
+		  GMimeMessage         * message)
+{
+	char * freeme = NULL;
+	int line_len;
+	const char * march;
+	const char * line_begin;
+	gboolean original_found_after_quoted = FALSE;
+	gboolean quoted_found = FALSE;
+
+	/* sanity clause */
+	g_return_if_fail (tm!=NULL);
+	g_return_if_fail (setme_errors!=NULL);
+	g_return_if_fail (setme_goodness!=NULL);
+	g_return_if_fail (body!=NULL);
+	g_return_if_fail (message!=NULL);
+
+	/* if it's not a reply, then top-posting check is moot */
+	if (g_mime_message_get_header (message, HEADER_REFERENCES) == NULL)
+		return;
+
+	/* walk through the body, looking for an original line after a quoted one */
+	march = body = freeme = strip_attribution_and_signature (body, message);
+	while ((get_next_token_run (march, '\n', &march, &line_begin, &line_len))) {
+		if (text_massager_is_quote_char (tm, (guchar)line_begin[0])) /* check for quoted */
+			quoted_found = TRUE;
+		else if (quoted_found) /* check for non-quoted after quoted */
+			original_found_after_quoted = TRUE;
+	}
+
+	/* did they put anything after the quote? */
+	if (quoted_found && !original_found_after_quoted) {
+		g_ptr_array_add (setme_errors, g_strdup(
+			_("WARNING: Reply seems to be top-posted.")));
+		*setme_goodness = MAX (*setme_goodness, WARN);
+	}
+
+	/* cleanup */
+	g_free (freeme);
+}
+
+
 /**
  * Check to see if the signature (if found) is within the McQuary limit of
  * four lines and 80 columns per line.
@@ -74,23 +161,31 @@ check_signature (GPtrArray      * setme_errors,
 	gint too_wide_qty = 0;
 	gint sig_line_qty = -1;
 	gint line_len = 0;
-	const gchar * line_begin = NULL;
+	const char * pch = NULL;
+	const char * line_begin = NULL;
+	char * sig_point = NULL;
+	SigType sig_type;
 
 	g_return_if_fail (setme_errors!=NULL);
 	g_return_if_fail (setme_goodness!=NULL);
 	g_return_if_fail (is_nonempty_string(body));
 
-	while (get_next_token_run (body, '\n', &body, &line_begin, &line_len))
+	sig_type = pan_find_signature_delimiter (body, &sig_point);
+
+	if (sig_type == SIG_NONSTANDARD)
 	{
-		/* find the sig delimiter */
-		if (sig_line_qty<0 && (line_len!=3 || strncmp(line_begin,"-- ",3)))
-			continue;
+		g_ptr_array_add (setme_errors, g_strdup(
+			_("WARNING: The signature marker should be \"-- \", not \"--\".")));
+		*setme_goodness = MAX (*setme_goodness, WARN);
+	}
 
+	pch = sig_point;
+	while (get_next_token_run (pch, '\n', &pch, &line_begin, &line_len))
+	{
 		++sig_line_qty;
-		if (line_len>80) {
-			++too_wide_qty;
-		}
 
+		if (line_len>80)
+			++too_wide_qty;
 	}
 
 	if (sig_line_qty == 0)
@@ -103,15 +198,13 @@ check_signature (GPtrArray      * setme_errors,
 	else if (sig_line_qty > 4)
 	{
 		g_ptr_array_add (setme_errors, g_strdup(
-			_("WARNING: signature is more than 4 lines long")));
+			_("WARNING: Signature is more than 4 lines long")));
 		*setme_goodness = MAX (*setme_goodness, WARN);
 	}
 
 	if (too_wide_qty != 0)
 	{
-		g_ptr_array_add (setme_errors, g_strdup_printf(
-			_("WARNING: %d lines too wide in signature"),
-			too_wide_qty));
+		g_ptr_array_add (setme_errors, g_strdup(_("WARNING: Signature is more than 80 characters wide.")));
 		*setme_goodness = MAX (*setme_goodness, WARN);
 	}
 }
@@ -143,7 +236,7 @@ check_wide_body (GPtrArray      * setme_errors,
 	if (too_wide_qty != 0)
 	{
 		g_ptr_array_add (setme_errors, g_strdup_printf(
-			_("WARNING: %d lines >80 characters in the message."),
+			_("WARNING: %d lines are more than 80 characters wide."),
 			too_wide_qty));
 		*setme_goodness = MAX (*setme_goodness, WARN);
 	}
@@ -256,125 +349,63 @@ check_mostly_quoted (GPtrArray      * setme_errors,
  *
  */
 static void
-check_all_quoted (GPtrArray      * setme_errors,
-                  GoodnessLevel  * setme_goodness,
-	          const gchar    * body,
-		  const Article  * article)
+check_all_quoted (const TextMassager   * tm,
+                  GPtrArray            * setme_errors,
+                  GoodnessLevel        * setme_goodness,
+	          const char           * body,
+		  GMimeMessage         * message)
 {
-	gchar * body2;
-	const gchar * pch;
-	gchar * attribution;
-	gint line_len;
-	const gchar * line_begin;
+	int line_len;
+	char * freeme;
+	const char * line_begin;
+	const char * march;
 
 	g_return_if_fail (is_nonempty_string(body));
 
-	/* (1) */
-	attribution = g_strdup (article_get_extra_header (article, PAN_ATTRIBUTION));
-	body2 = g_strdup (body);
-
-	/* (2) */
-	if (is_nonempty_string (attribution))
-		g_strdelimit (attribution, "\n", ' ');
-
-	/* (3) */
-	if (is_nonempty_string (attribution)) {
-		pch = pan_strstr (body2, attribution);
-		if (pch != NULL)
-		{	
-			char * src;
-			char * tgt;
-			int    attribution_len = strlen (attribution);
-			int    attribution_start_index = pch - body2;
-		
-			src = body2 + attribution_start_index + attribution_len;
-			tgt = body2 + attribution_start_index;
-			while (*src)
-				*tgt++ = *src++;
-			*tgt = '\0';
-		}
-	}
-	g_free (attribution);
-
-	/* (4) */
-	pch = body2;
-	while (get_next_token_run (pch, '\n', &pch, &line_begin, &line_len))
+	march = body = freeme = strip_attribution_and_signature (body, message);
+	while (get_next_token_run (march, '\n', &march, &line_begin, &line_len))
 	{
-		const gchar * pch = line_begin;
-		const gchar * end = line_begin + line_len;
-
-		/* check for sig reached */
-		if (line_len==3 && !strncmp(line_begin,"-- ",3))
-			break;
-
-		while (pch!=end && isspace((guchar)*pch))
-			++pch;
-		if (pch==end) /* blank line */
+		const char * march = line_begin;
+		const char * end = line_begin + line_len;
+		while (march!=end && isspace((guchar)*march))
+			++march;
+		if (march==end) /* blank line */
 			continue;
-
-		if (*pch != '>') { /* found new content */
-			g_free (body2);
+		if (!text_massager_is_quote_char (tm, (guchar)*march)) { /* found new content */
+			g_free (freeme);
 			return;
 		}
 	}
-	g_free (body2);
 
 	g_ptr_array_add (setme_errors, g_strdup(
 		_("ERROR: Message appears to have no new content.")));
 	*setme_goodness = MAX (*setme_goodness, REFUSE);
-}
 
-/**
- * Check the charset is correct for the body.
- * Since we've already g_converted at this point (article_get_body()),
- * body will be NULL if this is the case.
- */
-static void
-check_charset (GPtrArray      * setme_errors,
-               GoodnessLevel  * setme_goodness,
-               const gchar    * body,
-               const Article  * article)
-{
-	if (body == NULL)
-	{
-		/* FIXME: this should move to article.c */
-		const char * raw_body;
-		const char * user_charset;
-		const char * best_charset;
-	       
-		user_charset = article_get_extra_header (article, PAN_CHARSET);
-		raw_body = article_get_extra_header(article, PAN_BODY);
-		if (is_nonempty_string (raw_body))
-			best_charset = g_mime_charset_best (raw_body, strlen (raw_body));
-		else
-			best_charset = _("Unknown");
-
-		g_ptr_array_add (setme_errors, g_strdup_printf(
-			_("ERROR: message uses characters not specified in charset '%s' - possibly use '%s' instead?"), user_charset, best_charset));
-		*setme_goodness = MAX (*setme_goodness, REFUSE);
-	}
+	/* cleanup */
+	g_free (freeme);
 }
 
 static void
-check_body (GPtrArray       * setme_errors,
-	    GoodnessLevel   * setme_goodness,
-	    const Article   * article,
-	    const gchar     * body)
+check_body (const TextMassager    * tm,
+            GPtrArray             * setme_errors,
+	    GoodnessLevel         * setme_goodness,
+	    GMimeMessage          * message,
+	    const gchar           * body)
 {
-	gboolean posting = is_nonempty_string(article_get_extra_header (article, HEADER_NEWSGROUPS));
+	gboolean posting = is_nonempty_string (g_mime_message_get_header (message, HEADER_NEWSGROUPS));
 
 	g_return_if_fail (setme_goodness != NULL);
 	g_return_if_fail (setme_errors != NULL);
-	g_return_if_fail (article != NULL);
+	g_return_if_fail (message != NULL);
 
-	check_charset (setme_errors, setme_goodness, body, article);
-	if (posting && setme_errors->len == 0)
+	if (posting)
 	{
 		check_empty (setme_errors, setme_goodness, body);
 		check_wide_body (setme_errors, setme_goodness, body);
 		check_signature (setme_errors, setme_goodness, body);
 		check_mostly_quoted (setme_errors, setme_goodness, body);
-		check_all_quoted (setme_errors, setme_goodness, body, article);
+		check_all_quoted (tm, setme_errors, setme_goodness, body, message);
+		check_topposting (tm, setme_errors, setme_goodness, body, message);
 	}
 }
 
@@ -386,6 +417,7 @@ check_followup_to (GPtrArray        * setme_errors,
 		   Server           * server,
 		   const GPtrArray  * group_names)
 {
+	const PString poster = pstring_shallow ("poster", 6);
 	int i;
 
 	g_return_if_fail (setme_errors != NULL);
@@ -396,12 +428,13 @@ check_followup_to (GPtrArray        * setme_errors,
 	/* check to make sure all the groups exist */
 	for (i=0; i<group_names->len; ++i)
 	{
-		const char * name = (char*) g_ptr_array_index (group_names, i);
-		if (g_strcasecmp(name, "poster")
-			&& server_get_named_group(server,name)==NULL)
+		const PString name = pstring_shallow (g_ptr_array_index (group_names, i), -1);
+
+		if (!pstring_equal (&name, &poster)
+			&& server_get_named_group(server,&name)==NULL)
 		{
 			g_ptr_array_add (setme_errors, g_strdup_printf(
-				_("WARNING: Unknown group: \"%s\"."), name));
+				_("WARNING: Unknown group: \"%*.*s\"."), name.len, name.len, name.str));
 			*setme_goodness = MAX (*setme_goodness, WARN);
 		}
 	}
@@ -410,7 +443,7 @@ check_followup_to (GPtrArray        * setme_errors,
 	if (group_names->len > 5)
 	{
 		g_ptr_array_add (setme_errors, g_strdup(
-			_("WARNING: Following-Up to many groups.")));
+			_("WARNING: Following-Up too many groups.")));
 		*setme_goodness = MAX (*setme_goodness, WARN);
 	}
 }
@@ -448,25 +481,19 @@ check_groups (GPtrArray          * setme_errors,
 	/* check to make sure all the groups exist */
 	for (i=0; i<group_names->len; ++i)
 	{
-		const char * name = (char*) g_ptr_array_index (group_names, i);
-		Group * g = server_get_named_group (server, name);
+		const PString name = pstring_shallow (g_ptr_array_index (group_names, i), -1);
+		Group * g = server_get_named_group (server, &name);
 
 		if (g == NULL)
 		{
 			g_ptr_array_add (setme_errors, g_strdup_printf(
-				_("WARNING: Unknown group: \"%s\"."), name));
+				_("WARNING: Unknown group: \"%*.*s\"."), name.len, name.len, name.str));
 			*setme_goodness = MAX (*setme_goodness, WARN);
 		}
 		else if (group_is_read_only (g))
 		{
 			g_ptr_array_add (setme_errors, g_strdup_printf(
-				_("WARNING: Group \"%s\" is read-only."), name));
-			*setme_goodness = MAX (*setme_goodness, WARN);
-		}
-		else if (group_is_moderated (g))
-		{
-			g_ptr_array_add (setme_errors, g_strdup_printf(
-				_("INFORMATION: Please be aware that posts to \"%s\" will not appear until approved by the group's moderator."), name));
+				_("WARNING: Group \"%*.*s\" is read-only."), name.len, name.len, name.str));
 			*setme_goodness = MAX (*setme_goodness, WARN);
 		}
 	}
@@ -506,12 +533,12 @@ check_mail_addresses (GPtrArray      * setme_errors,
 {
 	guint address_count = 0;
 	InternetAddressList * addresses;
+	InternetAddressList * march;
 
-	addresses = internet_address_parse_string (addresses_str);
-
-	while (addresses)
+	march = addresses = internet_address_parse_string (addresses_str);
+	while (march)
 	{
-		InternetAddress * address = addresses->address;
+		InternetAddress * address = march->address;
 		char * from = internet_address_to_string (address, TRUE);
 		int val = gnksa_check_from (from, TRUE);
 
@@ -520,48 +547,56 @@ check_mail_addresses (GPtrArray      * setme_errors,
 			*setme_goodness = MAX (*setme_goodness, REFUSE);
 
 			g_ptr_array_add (setme_errors, g_strdup_printf (
-				_("ERROR: invalid address ``%s''"), from));
+				_("ERROR: invalid address \"%s\""), from));
 		}
 
-		address_count++;
-		internet_address_unref (address);
-		addresses = addresses->next;
+		++address_count;
+		march = march->next;
+
+		/* cleanup */
+		g_free (from);
 	}
 
+	/* cleanup */
+	internet_address_list_destroy (addresses);
 	return address_count;
 }
 
 
 void
-check_article (const Article    * article,
-               Server           * server,
-               GPtrArray        * appendme_errors,
-               GoodnessLevel    * setme_goodness)
+message_check (GMimeMessage        * message,
+               Server              * server,
+               GPtrArray           * appendme_errors,
+               GoodnessLevel       * setme_goodness)
 {
 	gchar * tmp;
 	int mail_qty;
 	int group_qty;
 	int author_qty;
 	const gchar * cpch;
+	gboolean is_html;
 	gboolean followup_to_set;
 	GoodnessLevel goodness = OKAY;
+	TextMassager * tm_default_values;
 
 	/* sanity check */
-	g_return_if_fail (article!=NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE(message));
 	g_return_if_fail (appendme_errors!=NULL);
 	g_return_if_fail (setme_goodness!=NULL);
 
 	/* check the subject... */
-	check_subject (appendme_errors, &goodness, article->subject);
+	check_subject (appendme_errors, &goodness, g_mime_message_get_subject(message));
 
 	/* check the body... */
-	tmp = article_get_body (article);
-	check_body (appendme_errors, &goodness, article, tmp);
+	tm_default_values = text_massager_new ();
+	tmp = g_mime_message_get_body (message, TRUE, &is_html);
+	check_body (tm_default_values, appendme_errors, &goodness, message, tmp);
 	g_free (tmp);
+	text_massager_free (tm_default_values);
 
 	/* check the optional followup-to... */
 	followup_to_set = FALSE;
-	cpch = article_get_extra_header (article, HEADER_FOLLOWUP_TO);
+	cpch = g_mime_message_get_header (message, HEADER_FOLLOWUP_TO);
 	if (is_nonempty_string (cpch))
 	{
 		GPtrArray * groups = get_nntp_rcpts (cpch);
@@ -578,7 +613,7 @@ check_article (const Article    * article,
 
 	/* check the groups... */
 	group_qty = 0;
-	cpch = article_get_extra_header (article, HEADER_NEWSGROUPS);
+	cpch = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
 	if (is_nonempty_string(cpch))
 	{
 		GPtrArray * groups = get_nntp_rcpts (cpch);
@@ -595,11 +630,8 @@ check_article (const Article    * article,
 	}
 
 	/* check the author... */
-	if (1) {
-		gchar author[512];
-		article_get_author_str (article, author, sizeof(author));
-		author_qty = check_mail_addresses (appendme_errors, &goodness, author);
-	}
+	cpch = g_mime_message_get_sender (message);
+	author_qty = check_mail_addresses (appendme_errors, &goodness, cpch);
 	if (author_qty!=1) {
 		/*
 		 * This is kind of silly: the From is controlled through a 
@@ -613,7 +645,7 @@ check_article (const Article    * article,
 	}
 
 	/* check the email recipients... */
-	cpch = article_get_extra_header (article, PAN_MAIL_TO);
+	cpch = g_mime_message_get_header (message, HEADER_TO);
 	mail_qty = check_mail_addresses (appendme_errors, &goodness, cpch);
 
 	/* one last error check */
