@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -33,17 +30,20 @@
 #include <pan/base/file-grouplist.h>
 #include <pan/base/file-headers.h>
 #include <pan/base/log.h>
+#include <pan/base/newsrc-port.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/server.h>
 #include <pan/base/serverlist.h>
 
+const PString INTERNAL_SERVER_NAME =  { "folders", 7, 0u };
+
 static void fire_groups_added          (Server*, Group**, gint group_qty);
 static void fire_groups_removed        (Server*, Group**, gint group_qty);
 
 /* this option for turning off saving exists for the regression tests,
  * which need to be able to turn it off */
-gint server_saving_enabled = TRUE;
+int server_saving_enabled = TRUE;
 
 /**
 ***  Server Object Life Cycle
@@ -61,13 +61,14 @@ server_constructor (Server              * server,
 	/* init the status-item bits */
 	debug1 (DEBUG_PAN_OBJECT, "server_constructor: %p", server);
 	server->posting = FALSE;
-	server->gen_msgid = FALSE;
 	server->need_auth = FALSE;
-	server->reserve_connection_for_bodies = TRUE;
-	server->address = NULL;
-	server->name = NULL;
-	server->username = NULL;
-	server->password = NULL;
+	server->use_newsrc = FALSE;
+	server->newsrc_imported = FALSE;
+	server->address = PSTRING_INIT;
+	server->name = PSTRING_INIT;
+	server->username = PSTRING_INIT;
+	server->password = PSTRING_INIT;
+	server->newsrc_filename = PSTRING_INIT;
 	server->port = 25;
 	server->max_connections = 2;
 	server->idle_secs_before_timeout = 180;
@@ -75,7 +76,7 @@ server_constructor (Server              * server,
 	server->_groups_dirty = 0;
 	server->_groups_loaded = 0;
 	server->_groups = g_ptr_array_new ();
-	server->_group_memchunk = memchunk_new (sizeof(Group), 100, FALSE);
+	server->_group_memchunk = memchunk_new (sizeof(Group), 1024, FALSE);
 }
 
 void
@@ -84,6 +85,10 @@ server_destructor (PanObject *obj)
 	guint i;
 	Server * server = SERVER(obj);
 
+	/* export the .newsrc file if necessary */
+	if (server->use_newsrc && pstring_is_set (&server->newsrc_filename))
+		newsrc_export (server, &server->newsrc_filename, FALSE);
+
 	/* clear out the server bits: groups */
 	server_save_if_dirty (server, NULL);
 	for (i=0; i!=server->_groups->len; ++i)
@@ -94,10 +99,11 @@ server_destructor (PanObject *obj)
 	server->_group_memchunk = NULL;
 
 	/* clear out the server bits: strings */
-	g_free (server->name);
-	g_free (server->address);
-	g_free (server->username);
-	g_free (server->password);
+	pstring_clear (&server->name);
+	pstring_clear (&server->address);
+	pstring_clear (&server->username);
+	pstring_clear (&server->password);
+	pstring_clear (&server->newsrc_filename);
 
 	/* clear out the parent */
 	pan_object_destructor (obj);
@@ -130,7 +136,7 @@ server_get_group_type (const Group * group)
        	type = 0;
 	if (group_is_subscribed (group))
 		type |= SERVER_GROUPS_SUBSCRIBED;
-	if (!group_is_subscribed (group))
+	else
 		type |= SERVER_GROUPS_UNSUBSCRIBED;
 
 	return type;
@@ -158,7 +164,7 @@ server_get_groups_type (const Group ** groups, gint qty)
 void
 server_ensure_groups_loaded (Server * server, ServerGroupsType type)
 {
-	gboolean load_needed = FALSE;
+	gboolean load_needed;
 	debug_enter ("server_ensure_groups_loaded");
 
 	/* sanity clause */
@@ -188,10 +194,11 @@ server_ensure_groups_loaded (Server * server, ServerGroupsType type)
 GPtrArray*
 server_get_groups (Server * server, ServerGroupsType type)
 {
-	gint i;
+	int i;
 	GPtrArray * a = g_ptr_array_new ();
 	debug_enter ("server_get_groups");
 
+	/* sanity clause */
 	g_return_val_if_fail (server_is_valid(server), a);
 	g_return_val_if_fail (type!=0, a);
 
@@ -234,14 +241,20 @@ server_set_group_type_dirty (Server * server, ServerGroupsType type)
 void
 server_save_grouplist_if_dirty (Server * server, StatusItem * status)
 {
+	GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	/* sanity clause */
 	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (server_saving_enabled);
 
+	/* save anything that's dirty */
+	g_static_mutex_lock (&mutex);
 	if (server->_groups_dirty)
 	{
 		file_grouplist_save (server, server->_groups_dirty, status);
 		server->_groups_dirty = FALSE;
 	}
+	g_static_mutex_unlock (&mutex);
 }
 
 void
@@ -290,19 +303,19 @@ server_alloc_new_group (Server * server)
 static int
 compare_name_ppgroup_name (const void* a, const void* b)
 {
-	return strcmp ((const gchar*)a, (*(const Group**)b)->name);
+	return pstring_compare ((const PString*)a, &(*(const Group**)b)->name);
 }
 
 static int
 compare_pgroup_ppgroup_name (const void* a, const void* b)
 {
-	return strcmp (((const Group*)a)->name, (*(const Group**)b)->name);
+	return pstring_compare (&((const Group*)a)->name, &(*(const Group**)b)->name);
 }
 
 static int
 compare_ppgroup_ppgroup_name (gconstpointer a, gconstpointer b, gpointer unused)
 {
-	return strcmp ((*(const Group**)a)->name, (*(const Group**)b)->name);
+	return pstring_compare (&(*(const Group**)a)->name, &(*(const Group**)b)->name);
 }
 
 
@@ -314,7 +327,7 @@ server_add_groups_impl (Server       * server,
                         GPtrArray    * fillme_not_used,
                         gboolean       initializing)
 {
-	gint i;
+	int i;
 	GHashTable * old;
 	GPtrArray * added;
 	debug_enter ("server_add_groups_impl");
@@ -323,39 +336,36 @@ server_add_groups_impl (Server       * server,
 	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (groups != NULL);
 	g_return_if_fail (group_qty >= 1);
+	for (i=0; i<group_qty; ++i) {
+		g_return_if_fail (groups[i] != NULL);
+		g_return_if_fail (groups[i]->server == server);
+	}
 
 	/* ensure the groups sets are loaded before adding to them */
 	if (1) {
-		ServerGroupsType type = 0;
+		ServerGroupsType type;
 		type = server_get_groups_type ((const Group **)groups, group_qty);
 		server_ensure_groups_loaded (server, type);
 	}
 
 	/* convert the existing grouplist to a hashtable */
-	old = g_hash_table_new (g_str_hash, g_str_equal);
+	old = g_hash_table_new (pstring_hash, pstring_equal);
 	for (i=0; i<server->_groups->len; ++i) {
 		Group * group = GROUP(g_ptr_array_index(server->_groups,i));
-		g_hash_table_insert (old, (gpointer)group->name, (gpointer)group);
+		g_hash_table_insert (old, &group->name, group);
 	}
 
 	/* try to add the new groups */
-	added = g_ptr_array_new ();
+	added = g_ptr_array_sized_new (group_qty);
 	for (i=0; i<group_qty; ++i)
 	{
 		Group * new_group = groups[i];
-		gboolean has_group = g_hash_table_lookup (old, new_group->name) != NULL;
+		gboolean has_group = g_hash_table_lookup (old, &new_group->name) != NULL;
 
 		if (!has_group)
 		{
-			/* update the group's server field */
-			if (new_group->server!=server) {
-				if (new_group->server!=NULL)
-					server_remove_groups (new_group->server, &new_group, 1);
-				new_group->server = server;
-			}
-
 			/* add the group */
-			g_hash_table_insert (old, (gpointer)new_group->name, new_group);
+			g_hash_table_insert (old, &new_group->name, new_group);
 			g_ptr_array_add (added, new_group);
 		}
 		else
@@ -368,8 +378,6 @@ server_add_groups_impl (Server       * server,
 	/* if any change was made, update the states */
 	if (added->len != 0)
 	{
-		ServerGroupsType type;
-
 		/* update the groups array */
 		pan_hash_to_ptr_array  (old, server->_groups);
 		g_ptr_array_sort_with_data (server->_groups, compare_ppgroup_ppgroup_name, NULL);
@@ -377,7 +385,7 @@ server_add_groups_impl (Server       * server,
 		if (!initializing)
 		{
 			/* mark the appropriate sets as dirty */
-			type = server_get_groups_type ((const Group**)added->pdata, added->len);
+			ServerGroupsType type = server_get_groups_type ((const Group**)added->pdata, added->len);
 			server_set_group_type_dirty (server, type);
 
 			/* let listeners know which were added */
@@ -435,9 +443,7 @@ server_remove_groups (Server * server, Group ** groups, int qty)
 	}
 
 	/* ensure the groupset is loaded */
-	type = 0;
-	for (i=0; i<qty; ++i)
-		type |= server_get_group_type (groups[i]);
+	type = server_get_groups_type ((const Group **)groups, qty);
 	server_ensure_groups_loaded (server, type);
 
 	/* find the groups to remove */
@@ -472,9 +478,52 @@ server_remove_groups (Server * server, Group ** groups, int qty)
 	debug_exit ("server_remove_groups");
 }
 
+void
+server_remove_unused_groups (Server * server, Group ** new_groups, gint qty)
+{
+	guint i;
+	ServerGroupsType type;
+	GHashTable * old = NULL;
+	debug_enter ("server_remove_unused_groups");
+
+	/* sanity clause */
+	g_return_if_fail (server_is_valid (server));
+	g_return_if_fail (qty > 0);
+	for (i=0; i<qty; ++i) {
+		g_return_if_fail (new_groups[i] != NULL);
+		g_return_if_fail (new_groups[i]->server == server);
+	}
+
+	/* ensure the groupset is loaded */
+	type = server_get_groups_type ((const Group **)new_groups, qty);
+	server_ensure_groups_loaded (server, type);
+
+	/* convert the existing grouplist to a hashtable */
+	old = g_hash_table_new (pstring_hash, pstring_equal);
+	for (i=0; i<server->_groups->len; ++i) {
+		Group * group = GROUP(g_ptr_array_index(server->_groups,i));
+		g_hash_table_insert (old, &group->name, group);
+	}
+
+	/* remove all new groups */
+	for (i=0; i<qty; i++)
+		g_hash_table_remove (old, &new_groups[i]->name);
+
+	/* if any groups left, remove them */
+	if (g_hash_table_size (old) > 0) {
+		GPtrArray * old_groups = g_ptr_array_new ();
+		pan_hash_to_ptr_array (old, old_groups);
+		server_remove_groups (server, (Group **)old_groups->pdata, old_groups->len);
+		g_ptr_array_free (old_groups, TRUE);
+	}
+
+	/* cleanup */
+	g_hash_table_destroy (old);
+}
+
 Group*
 server_get_named_group_in_type (Server            * server,
-                                const char        * name,
+                                const PString     * name,
                                 ServerGroupsType    set)
 {
 	int index;
@@ -484,7 +533,7 @@ server_get_named_group_in_type (Server            * server,
 
 	/* sanity clause */
 	g_return_val_if_fail (server_is_valid (server), NULL);
-	g_return_val_if_fail (is_nonempty_string(name), NULL);
+	g_return_val_if_fail (pstring_is_set (name), NULL);
 	g_return_val_if_fail (set!=0, NULL);
 
 	/* find where to add this group */
@@ -515,7 +564,7 @@ server_get_named_group_in_type (Server            * server,
 
 Group*
 server_get_named_group (Server        * server,
-	                const char    * name)
+	                const PString * name)
 {
 	return server_get_named_group_in_type (server, name, SERVER_GROUPS_ALL);
 }
@@ -524,7 +573,7 @@ gboolean
 server_is_valid (const Server * server)
 {
 	g_return_val_if_fail (server!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string (server->name), FALSE);
+	g_return_val_if_fail (pstring_is_valid (&server->name), FALSE);
 	g_return_val_if_fail (server->_group_memchunk!=NULL, FALSE);
 	g_return_val_if_fail (server->_groups!=NULL, FALSE);
 	/* arbitrary fishing for corruption */
@@ -534,11 +583,48 @@ server_is_valid (const Server * server)
 	return TRUE;
 }
 
-const gchar * server_get_name (const Server * server)
+const char*
+server_get_name (const Server * server)
 {
 	g_return_val_if_fail (server_is_valid (server), "");
 
-	return server->name;
+	return server->name.str;
+}
+
+void
+server_set_name (Server          * server,
+                 const PString   * new_name,
+                 GError         ** error)
+{
+	gboolean ok = TRUE;
+	PString my_new_name;
+
+	/* sanity clause */
+	g_return_if_fail (server != NULL);
+	g_return_if_fail (pstring_is_valid (new_name));
+	g_return_if_fail (error==NULL || *error==NULL);
+
+	/* create a stripped copy of new_name */
+	my_new_name = pstring_strstrip_shallow (new_name);
+
+	/* make sure a name's specified */
+	if (!pstring_is_set (&my_new_name)) {
+		if (error != NULL)
+			*error = g_error_new (g_quark_from_string("server"), 1, _("No name specified!"));
+		ok = FALSE;
+	}
+
+	if (ok)
+	{
+		/* if name is changing -- not just being initialized -- tell file_headers */
+		if (pstring_is_set(&server->name) && !pstring_equal (&server->name, &my_new_name)) {
+			file_headers_server_name_changed (&server->name, &my_new_name);
+			file_grouplist_server_name_changed (&server->name, &my_new_name);
+		}
+
+		/* update the server's name */
+		pstring_copy (&server->name, &my_new_name);
+	}
 }
 
 void
