@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,19 +15,20 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __SERVER_H__
 #define __SERVER_H__
 
 #include <time.h>
-#include <glib.h>
+#include <glib/gtypes.h>
+#include <glib/garray.h>
 
 #include <gmime/memchunk.h>
 
 #include <pan/base/pan-callback.h>
 #include <pan/base/pan-object.h>
+#include <pan/base/pstring.h>
 #include <pan/base/status-item.h>
 
 #define SERVER(a) ((Server*)a)
@@ -43,33 +42,35 @@ typedef enum
 }
 ServerGroupsType;
 
-typedef struct
+typedef struct _Server Server;
+
+struct _Server
 {
 	/* Parent */
 	PanObject parent;
 
 	/*Fields*/
 	guint posting              : 1;  /* is posting allowed? */
-	guint gen_msgid            : 1;
 	guint need_auth            : 1;  /* need authorization ? */
-	guint reserve_connection_for_bodies     : 1;
+	guint use_newsrc           : 1;  /* import/export .newsrc files? */
+	guint newsrc_imported      : 1;  /* have we already imported it? */
 	gint8 max_connections;           /* max connections to server */
         gint16 idle_secs_before_timeout;
 	gint port;	                 /* port number.  This is usually 119 for NNTP. */
 	time_t last_newgroup_list_time;
-	char * address;                  /* server net address */
-	char * name;	                 /* pretty name, i.e. Erols */
-	char * username;                 /* auth username */
-	char * password;                 /* auth password */
+	PString newsrc_filename;         /* import/export this */
+	PString address;                 /* server net address */
+	PString username;                /* auth username */
+	PString password;                /* auth password */
+	PString name;                    /* pretty name, i.e. Erols */
 
 	ServerGroupsType _groups_dirty;
 	ServerGroupsType _groups_loaded;
 	MemChunk * _group_memchunk;
 	GPtrArray * _groups;             /* don't access directly; use server_get_groups instead */
-}
-Server;
+};
 
-#define INTERNAL_SERVER_NAME "folders"
+extern const PString INTERNAL_SERVER_NAME;
 
 /**
 ***  PROTECTED
@@ -87,7 +88,11 @@ Server*       server_new (void);
 
 gboolean      server_is_valid            (const Server*);
 
-const gchar * server_get_name            (const Server*);
+const char *  server_get_name            (const Server*);
+
+void          server_set_name            (Server          * server,
+                                          const PString   * name,
+                                          GError         ** error);
 
 
 /**
@@ -99,10 +104,10 @@ struct _Group;
 struct _Group* server_alloc_new_group          (Server            * server);
 
 struct _Group* server_get_named_group          (Server            * server,
-                                                const char        * name);
+                                                const PString     * name);
 
 struct _Group* server_get_named_group_in_type  (Server            * server,
-                                                const char        * name,
+                                                const PString     * name,
                                                 ServerGroupsType    type);
 
 
@@ -118,12 +123,12 @@ void       server_add_groups             (Server            * server,
                                           GPtrArray         * fillme_used,
                                           GPtrArray         * fillme_not_used);
 
-gboolean    server_add_group             (Server * server, struct _Group * group);
-
 void        server_remove_groups          (Server * server, struct _Group ** groups, gint qty);
 
 void        server_destroy_groups         (Server * server, struct _Group ** groups, gint qty);
 
+void        server_remove_unused_groups   (Server * server, struct _Group ** new_groups, gint qty);
+
 void        server_ensure_groups_loaded   (Server * server, ServerGroupsType type);
 
 GPtrArray*  server_get_groups             (Server * server, ServerGroupsType type);
