@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -29,10 +26,11 @@
 
 #include <gmime/gmime.h>
 
+#include <pan/base/serverlist.h>
+#include <pan/base/article.h>
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/util-mime.h>
-#include <pan/base/util-file.h>
 
 /***
 **** YENC
@@ -177,7 +175,7 @@ yenc_parse_begin_line (const char  * b,
 	ll = __yenc_extract_tag_val_int (b, YENC_TAG_LINE);
 	g_return_val_if_fail (ll != 0, -1);
 
-	// part is optional
+	/* part is optional */
 	part_num = __yenc_extract_tag_val_int (b, YENC_TAG_PART);
 	
 	a_sz = __yenc_extract_tag_val_int( b, YENC_TAG_SIZE );
@@ -205,7 +203,7 @@ yenc_parse_begin_line (const char  * b,
  * of the part inside the end file
  */
 static int
-yenc_parse_part_line (const char * b, int *begin_offset, int *end_offset)
+yenc_parse_part_line (const char * b, guint *begin_offset, guint *end_offset)
 {
 	int bg;
 	int end;
@@ -226,35 +224,20 @@ yenc_parse_part_line (const char * b, int *begin_offset, int *end_offset)
 	return 0;
 }
 
-
-#if 0
-static gboolean
-is_yenc_line (const char * line, int len)
-{
-	if (len != YENC_FULL_LINE_LEN || len != YENC_HALF_LINE_LEN) {
-		if (len != YENC_FULL_LINE_LEN + 1 || YENC_HALF_LINE_LEN + 1)
-			return FALSE;
-	}
-	
-	// TODO if +1 than line length check for escape chars at the end
-	return TRUE;
-}
-#endif
-
 /**
  * yenc_is_beginning
  * @param line line to test & see if it's the beginning of a yenc block
  * @return true if it is, false otherwise
  */
 static gboolean
-yenc_is_beginning_line (const gchar * line)
+yenc_is_beginning_line (const char * line)
 {
 	return !strncmp (line, YENC_MARKER_BEGIN, YENC_MARKER_BEGIN_LEN) &&
 	       !yenc_parse_begin_line (line, NULL, NULL, NULL, NULL);
 }
 
 static gboolean
-yenc_is_part_line (const gchar * line)
+yenc_is_part_line (const char * line)
 {
 	return !strncmp (line, YENC_MARKER_PART, YENC_MARKER_PART_LEN) &&
 	       !yenc_parse_part_line (line, NULL, NULL);
@@ -266,7 +249,7 @@ yenc_is_part_line (const gchar * line)
  * @return true if it is, false otherwise
  */
 static gboolean
-yenc_is_ending_line (const gchar * line)
+yenc_is_ending_line (const char * line)
 {
 	return !strncmp (line, YENC_MARKER_END, YENC_MARKER_END_LEN) &&
 	       !yenc_parse_end_line (line, NULL, NULL, NULL, NULL);
@@ -277,41 +260,40 @@ yenc_is_ending_line (const gchar * line)
 **** UU
 ***/
 
-/**
- * parse_uu_begin_line
- * @param line the line to check for "begin filename mode"
- * @param filename if parse is successful, is set with the
- *        starting character of the filename.
- * @param mode if parse is successful, mode is set with the
- *        mode requested by the line.
- * @return 0 on success, -1 on failure
- */
 static int
-uu_parse_begin_line (const gchar  * b,
-                     const gchar ** file,
-                            gint  * mode)
+uu_parse_begin_line (const PString    * begin,
+                     char            ** setme_filename,
+                     gulong           * setme_mode)
 {
-	gint m = 0;
+	int retval;
+	char * end;
+	PString tmp;
+	gulong mode;
 
-	/* skip to the permissions */
-	while (*b==' ') ++b;
-	if (!isdigit((guchar)*b)) return -1;
+	g_return_val_if_fail (pstring_is_set (begin), -1);
 
-	/* parse the permissions */
-	while (isdigit((guchar)*b)) {
-		m = m*8 + (*b-'0');
-		++b;
-	};
+	/* skip past the "begin " */
+	tmp = pstring_substr_shallow (begin, begin->str+6, NULL);
+	mode = strtoul (tmp.str, &end, 8);
+	if (end==NULL || end-tmp.str<3) /* must have at least 3 octal digits */
+		mode = 0ul;
 
-	/* skip to the filename */
-	if (*b!=' ') return -1;
-	while (*b==' ') ++b;
-	if (*b < ' ') return -1;
+	tmp = pstring_substr_shallow (&tmp, end, NULL); /* skip past the permissions */
+	end = pstring_strchr (&tmp, '\n');              /* remove linefeed, if any */
+	tmp = pstring_substr_shallow (&tmp, NULL, end); 
+	tmp = pstring_strstrip_shallow (&tmp);          /* remove whitespace */
 
-	/* return the results */
-	if (mode!=NULL) *mode = m;
-	if (file!=NULL) *file = b;
-	return 0;
+	if (mode==0 || !pstring_is_set(&tmp))
+		retval = -1;
+	else {
+		if (setme_mode != NULL)
+			*setme_mode = mode;
+		if (setme_filename != NULL)
+			*setme_filename = g_strndup (tmp.str, tmp.len);
+		retval = 0;
+	}
+
+	return retval;
 }
 
 /**
@@ -320,50 +302,25 @@ uu_parse_begin_line (const gchar  * b,
  * @return true if it is, false otherwise
  */
 static gboolean
-uu_is_beginning_line (const gchar * line)
+uu_is_beginning_line (const PString * line)
 {
-	return line!=NULL
-	 	&& (line[0]=='b' || line[0]=='B')
-	 	&& (line[1]=='e' || line[1]=='E')
-	 	&& (line[2]=='g' || line[2]=='G')
-	 	&& (line[3]=='i' || line[3]=='I')
-	 	&& (line[4]=='n' || line[4]=='N')
-		&& line[5]==' '
-		&& !uu_parse_begin_line (line+6, NULL, NULL);
+	return pstring_is_set(line)
+		&& line->len>5
+		&& (!memcmp(line->str, "begin ", 6) || !memcmp(line->str, "BEGIN ", 6))
+		&& !uu_parse_begin_line (line, NULL, NULL);
 }
 
-
 /**
  * uu_is_ending
  * @param line line to test & see if it's the end of a uu-encoded block
  * @return true if it is, false otherwise
  */
 static gboolean
-uu_is_ending_line (const gchar * line)
+uu_is_ending_line (const char * line)
 {
 	return line!=NULL && !g_strncasecmp(line,"end",3) && !strstr(line,"cut") && !strstr(line,"CUT");
 }
 
-static void
-uu_get_file_info (const gchar       * begin,
-                  gchar            ** setme_filename,
-                  gulong            * setme_mode)
-{
-	gchar * filename;
-	const gchar * end;
-
-	g_return_if_fail (uu_is_beginning_line(begin));
-	g_return_if_fail (setme_filename != NULL);
-	g_return_if_fail (setme_mode != NULL);
-
-	*setme_mode = strtol (begin+6, NULL, 8);
-	
-	begin += 10;
-	end = strchr (begin, '\n');
-	filename = g_strndup (begin, end-begin);
-	g_strstrip (filename);
-	*setme_filename = filename;
-}
 
 static gboolean
 is_uu_line (const char * line, int len)
@@ -415,12 +372,12 @@ static guint
 stream_readln (GMimeStream *stream, GByteArray *line, off_t* startpos)
 {
 	char linebuf[1024];
-	ssize_t len;
+	gssize len;
 
 	/* sanity clause */
-	g_return_val_if_fail (stream!=NULL, FALSE);
-	g_return_val_if_fail (line!=NULL, FALSE);
-	g_return_val_if_fail (startpos!=NULL, FALSE);
+	g_return_val_if_fail (stream!=NULL, 0u);
+	g_return_val_if_fail (line!=NULL, 0u);
+	g_return_val_if_fail (startpos!=NULL, 0u);
 
 	/* where are we now? */
 	*startpos = g_mime_stream_tell (stream);
@@ -471,16 +428,15 @@ separate_encoded_parts (GMimeStream * istream,
 	gboolean yenc_looking_for_part_line = FALSE;
 	off_t linestart_pos = 0;
 	off_t sub_begin = 0;
-	off_t sub_end = 0;
 	guint line_len;
+	guint valid_encoded_lines_in_part;
 
 	/* sanity clause */
 	g_return_if_fail (istream!=NULL);
 	g_return_if_fail (appendme_file_parts!=NULL);
 
 	sub_begin = 0;
-	sub_end = 0;
-	line = g_byte_array_new ();
+	line = g_byte_array_sized_new (4096);
 	while ((line_len = stream_readln (istream, line, &linestart_pos)))
 	{
 		char * line_str = (char*) line->data;
@@ -494,7 +450,9 @@ separate_encoded_parts (GMimeStream * istream,
 		{
 			case ENC_PLAIN:
 			{
-				if (uu_is_beginning_line (line_str))
+				const PString line_pstr = pstring_shallow (line_str, line_len);
+
+				if (uu_is_beginning_line (&line_pstr))
 				{
 					gulong mode = 0ul;
 					char * decoded_filename = NULL;
@@ -504,13 +462,14 @@ separate_encoded_parts (GMimeStream * istream,
 					{
 						GMimeStream * stream = g_mime_stream_substream (istream, sub_begin, linestart_pos);
 						g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cur->cat), stream);
-						g_object_unref (stream);
-                        			g_ptr_array_add (appendme_file_parts, cur);
+						g_mime_stream_unref (stream);
+						g_ptr_array_add (appendme_file_parts, cur);
 						cur = NULL;
 					}
 
 					sub_begin = linestart_pos;
-					uu_get_file_info (line_str, &decoded_filename, &mode);
+					valid_encoded_lines_in_part = 0u;
+					uu_parse_begin_line (&line_pstr, &decoded_filename, &mode);
 
 					cur = g_new0 (EncTempFilePart, 1);
 					cur->type = type = ENC_UU;
@@ -524,12 +483,13 @@ separate_encoded_parts (GMimeStream * istream,
 					{
 						GMimeStream * stream = g_mime_stream_substream (istream, sub_begin, linestart_pos);
 						g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cur->cat), stream);
-						g_object_unref (stream);
-                        			g_ptr_array_add (appendme_file_parts, cur);
+						g_mime_stream_unref (stream);
+						g_ptr_array_add (appendme_file_parts, cur);
 						cur = NULL;
 					}
 
 					sub_begin = linestart_pos;
+					valid_encoded_lines_in_part = 0u;
 
 					cur = g_new0 (EncTempFilePart, 1);
 					cur->type = type = ENC_YENC;
@@ -545,6 +505,8 @@ separate_encoded_parts (GMimeStream * istream,
 				else if (cur == NULL)
 				{
 					sub_begin = linestart_pos;
+					valid_encoded_lines_in_part = 0u;
+
 					cur = g_new0 (EncTempFilePart, 1);
 					cur->type = type = ENC_PLAIN;
 					cur->cat = g_mime_stream_cat_new ();
@@ -560,7 +522,7 @@ separate_encoded_parts (GMimeStream * istream,
 						sub_begin = linestart_pos;
 					stream = g_mime_stream_substream (istream, sub_begin, linestart_pos+line_len);
 					g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cur->cat), stream);
-					g_object_unref (stream);
+					g_mime_stream_unref (stream);
                         		g_ptr_array_add (appendme_file_parts, cur);
 
 					cur = NULL;
@@ -573,7 +535,7 @@ separate_encoded_parts (GMimeStream * istream,
 					{
 						GMimeStream * stream = g_mime_stream_substream (istream, sub_begin, linestart_pos);
 						g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cur->cat), stream);
-						g_object_unref (stream);
+						g_mime_stream_unref (stream);
 					}
 
 					sub_begin = -1;
@@ -581,8 +543,13 @@ separate_encoded_parts (GMimeStream * istream,
 				else if (sub_begin == -1)
 				{
 					/* looks like they decided to start using uu lines again. */
+					++valid_encoded_lines_in_part;
 					sub_begin = linestart_pos;
 				}
+				else
+				{
+					++valid_encoded_lines_in_part;
+				}
 				break;
 			}
 			case ENC_YENC:
@@ -591,7 +558,7 @@ separate_encoded_parts (GMimeStream * istream,
 				{
 					GMimeStream * stream = g_mime_stream_substream (istream, sub_begin, linestart_pos+line_len);
 					g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cur->cat), stream);
-					g_object_unref (stream);
+					g_mime_stream_unref (stream);
 					yenc_parse_end_line (line_str, &cur->y_size, NULL, &cur->y_pcrc, &cur->y_crc);
                         		g_ptr_array_add (appendme_file_parts, cur);
 
@@ -602,6 +569,11 @@ separate_encoded_parts (GMimeStream * istream,
 				{
 					yenc_parse_part_line (line_str, &cur->y_offset_begin, &cur->y_offset_end);
 					yenc_looking_for_part_line = FALSE;
+					++valid_encoded_lines_in_part;
+				}
+				else
+				{
+					++valid_encoded_lines_in_part;
 				}
 				break;
 			}
@@ -615,8 +587,13 @@ separate_encoded_parts (GMimeStream * istream,
 		{
 			GMimeStream * stream = g_mime_stream_substream (istream, sub_begin, linestart_pos);
 			g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cur->cat), stream);
-			g_object_unref (stream);
+			g_mime_stream_unref (stream);
 		}
+
+		/* just in case someone started with "yenc" or "begin 644 asf" in a text message to fuck with unwary newsreaders */
+		if (valid_encoded_lines_in_part < 10u)
+			cur->type = ENC_PLAIN;
+
 		g_ptr_array_add (appendme_file_parts, cur);
 		cur = NULL;
 		type = ENC_PLAIN;
@@ -630,7 +607,7 @@ guess_part_type_from_filename (const char   * filename,
                                const char  ** setme_type,
                                const char  ** setme_subtype)
 {
-	const static struct {
+	static const struct {
 		const char * suffix;
 		const char * type;
 		const char * subtype;
@@ -665,7 +642,7 @@ guess_part_type_from_filename (const char   * filename,
 		{ ".xsl",   "text",         "xml" },
 		{ ".zip",   "application",  "zip" }
 	};
-	const static int suffix_qty = G_N_ELEMENTS (suffixes);
+	static const int suffix_qty = G_N_ELEMENTS (suffixes);
 	const char * suffix;
 
 	/* zero out the return values */
@@ -694,6 +671,19 @@ guess_part_type_from_filename (const char   * filename,
 	}
 }
 
+static void
+free_yenc_info (gpointer p)
+{
+	YencInfo * y;
+
+	g_return_if_fail (p!=NULL);
+
+	y = (YencInfo*) p;
+	g_free (y->filename);
+	g_free (y);
+}
+
+
 static void
 handle_inline_encoded_data (GMimeObject **part)
 {
@@ -729,12 +719,13 @@ handle_inline_encoded_data (GMimeObject **part)
 	content = (GMimeDataWrapper *) g_mime_part_get_content_object (GMIME_PART (*part));
 	if (!content)
 		return;
-	
+
 	stream = g_mime_data_wrapper_get_stream (content);
 	g_mime_stream_reset (stream);
 	istream = g_mime_stream_buffer_new (stream, GMIME_STREAM_BUFFER_BLOCK_READ);
-	g_object_unref (stream);
-	
+	g_mime_stream_unref (stream);
+	g_object_unref (content);
+
 	parts = g_ptr_array_new ();
 	separate_encoded_parts (istream, parts);
 	g_mime_stream_reset (istream);
@@ -752,10 +743,8 @@ handle_inline_encoded_data (GMimeObject **part)
 			GMimePart *subpart = NULL;
 			GMimeStream * filter_stream;
 			GMimeStream * substream;
-			GMimeFilter * filter;
 			const char * type = NULL;
 			const char * subtype = NULL;
-			char *cid;
 			
 			/* create the new subpart's stream */
 
@@ -766,22 +755,16 @@ handle_inline_encoded_data (GMimeObject **part)
 				guess_part_type_from_filename (enc_part->decoded_filename, &type, &subtype);
 				
 				subpart = g_mime_part_new_with_type (type, subtype);
-				
+
 				/* set part's attributes */
 				g_mime_part_set_filename (subpart, enc_part->decoded_filename);
-				cid = g_strdup_printf ("<%s>", enc_part->decoded_filename);
-				g_mime_part_set_content_id (subpart, cid);
-				g_free (cid);
-				
+
 				filter_stream = g_mime_stream_filter_new_with_stream (substream);
-				
-				filter = g_mime_filter_basic_new_type (GMIME_FILTER_BASIC_UU_DEC);
-				g_mime_stream_filter_add (GMIME_STREAM_FILTER (filter_stream), filter);
-				content = g_mime_data_wrapper_new_with_stream (filter_stream,
-									       GMIME_PART_ENCODING_DEFAULT);
-				g_object_unref (filter_stream);
-				
+				g_mime_stream_filter_add (GMIME_STREAM_FILTER (filter_stream), g_mime_filter_basic_new_type (GMIME_FILTER_BASIC_UU_DEC));
+				content = g_mime_data_wrapper_new_with_stream (filter_stream, GMIME_PART_ENCODING_DEFAULT);
 				g_mime_part_set_content_object (subpart, content);
+				g_object_unref (content);
+				g_mime_stream_unref (filter_stream);
 			}
 			else if (enc_part->type == ENC_YENC)
 			{
@@ -792,9 +775,6 @@ handle_inline_encoded_data (GMimeObject **part)
 				
 				/* set part's attributes */
 				g_mime_part_set_filename (subpart, enc_part->decoded_filename);
-				cid = g_strdup_printf ("<%s>", enc_part->decoded_filename);
-				g_mime_part_set_content_id (subpart, cid);
-				g_free (cid);
 				
 				g_mime_part_set_content_disposition (subpart, "inline");
 				
@@ -802,6 +782,7 @@ handle_inline_encoded_data (GMimeObject **part)
 				content = g_mime_data_wrapper_new_with_stream (substream,
 									       GMIME_PART_ENCODING_DEFAULT);
 				g_mime_part_set_content_object (subpart, content);
+				g_object_unref (content);
 				
 				yenc_info = g_new0 (YencInfo, 1);
 				yenc_info->filename = g_strdup (enc_part->decoded_filename);
@@ -813,7 +794,7 @@ handle_inline_encoded_data (GMimeObject **part)
 				yenc_info->crc = enc_part->y_crc;
 				yenc_info->pcrc = enc_part->y_pcrc;
 				yenc_info->size = enc_part->y_size;
-				GMIME_OBJECT (subpart)->user_data = yenc_info;
+				g_object_set_data_full (G_OBJECT(subpart), "yenc", yenc_info, free_yenc_info);
 			}
 			else
 			{
@@ -823,6 +804,7 @@ handle_inline_encoded_data (GMimeObject **part)
 									       GMIME_PART_ENCODING_8BIT);
 				
 				g_mime_part_set_content_object (subpart, content);
+				g_object_unref (content);
 			}
 			
 			g_mime_multipart_add_part (GMIME_MULTIPART (multipart), GMIME_OBJECT (subpart));
@@ -834,11 +816,11 @@ handle_inline_encoded_data (GMimeObject **part)
 		*part = GMIME_OBJECT (multipart);
 	}
 
-	g_object_unref (istream);
+	g_mime_stream_unref (istream);
 
 	for (i=0; i<parts->len; ++i) {	
 		EncTempFilePart * enc_part = parts->pdata[i];
-		g_object_unref (enc_part->cat);
+		g_mime_stream_unref (enc_part->cat);
 		g_free (enc_part->decoded_filename);
 		g_free (enc_part);
 	}
@@ -857,7 +839,7 @@ handle_inline_encoded_data (GMimeObject **part)
  *
  * Returns a partial mime part.
  */
-GMimeMessagePartial*
+static GMimeMessagePartial*
 pan_fake_partial (GMimeMessage  * parent,
                   GMimeStream   * content,
                   const char    * id,
@@ -889,29 +871,35 @@ pan_fake_partial (GMimeMessage  * parent,
 		/* cat stream holds headers + body */
 		cat = g_mime_stream_cat_new ();
 		g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cat), stream);
-		g_object_unref (stream);
+		g_mime_stream_unref (stream);
 		g_mime_stream_cat_add_source (GMIME_STREAM_CAT(cat), content);
 
 		/* override the content variable s.t. headers get written too */
 		partial = g_mime_message_partial_new (id, number, total);
 		wrapper = g_mime_data_wrapper_new_with_stream (cat, GMIME_PART_ENCODING_DEFAULT);
 		g_mime_part_set_content_object (GMIME_PART(partial), wrapper);
-		g_object_unref (cat);
+		g_object_unref (wrapper);
+		g_mime_stream_unref (cat);
 	}
 	else
 	{
 		partial = g_mime_message_partial_new (id, number, total);
 		wrapper = g_mime_data_wrapper_new_with_stream (content, GMIME_PART_ENCODING_DEFAULT);
 		g_mime_part_set_content_object (GMIME_PART(partial), wrapper);
+		g_object_unref (wrapper);
 	}
 
 	return partial;
 }
 
-static GMimeMessage*
-construct_message_impl (const char * message_id, GMimeStream ** istreams, int qty)
+
+
+GMimeMessage*
+pan_g_mime_parser_construct_message (GMimeStream  ** istreams,
+                                     int             qty)
 {
 	int i;
+	const char * message_id = "Foo <bar@mum>";
 	GMimeMessage * retval = NULL;
 	GMimeMessage ** messages = NULL;
 	GMimeParser * parser;
@@ -920,10 +908,12 @@ construct_message_impl (const char * message_id, GMimeStream ** istreams, int qt
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
 	g_return_val_if_fail (istreams!=NULL, NULL);
 	g_return_val_if_fail (qty>=1, NULL);
+	for (i=0; i<qty; ++i)
+		g_return_val_if_fail (GMIME_IS_STREAM(istreams[i]), NULL);
 
 	/* build the GMimeMessages */
 	parser = g_mime_parser_new ();
-	messages = g_new0 (GMimeMessage*, qty);
+	messages = g_newa (GMimeMessage*, qty);
 	for (i=0; i<qty; ++i) {
 		g_mime_parser_init_with_stream (parser, istreams[i]);
 		messages[i] = g_mime_parser_construct_message (parser);
@@ -937,8 +927,8 @@ construct_message_impl (const char * message_id, GMimeStream ** istreams, int qt
 	}
 	else
 	{
-		gpointer * unref = g_new (gpointer, qty);
-		GMimeMessagePartial ** partials  = g_new (GMimeMessagePartial*, qty);
+		gpointer * unref = g_newa (gpointer, qty);
+		GMimeMessagePartial ** partials  = g_newa (GMimeMessagePartial*, qty);
 
 		/* make an array of message/partials that to reconstruct a single message */
 		for (i=0; i<qty; ++i)
@@ -958,11 +948,13 @@ construct_message_impl (const char * message_id, GMimeStream ** istreams, int qt
 
 				/* make a partial */
 				message = messages[i];
-				wrapper = (GMimeDataWrapper *) g_mime_part_get_content_object (GMIME_PART(message->mime_part));
-				stream = g_mime_data_wrapper_get_stream (wrapper);
+				wrapper = g_mime_part_get_content_object (GMIME_PART(message->mime_part));
+				stream = g_mime_data_wrapper_get_stream ((GMimeDataWrapper*)wrapper);
 				partial = pan_fake_partial (message, stream, message_id, i+1, qty);
 				g_mime_stream_reset (stream);
-				g_object_unref (stream);
+
+				g_mime_stream_unref (stream);
+				g_object_unref (wrapper);
 				g_object_unref (message);
 
 				unref[i] = partial;
@@ -977,8 +969,6 @@ construct_message_impl (const char * message_id, GMimeStream ** istreams, int qt
 		/* cleanup */
 		for (i=0; i<qty; ++i)
 			g_object_unref (unref[i]);
-		g_free (partials);
-		g_free (unref);
 	}
 
 	/* pick out yenc and uuenc data in text/plain messages */
@@ -986,71 +976,133 @@ construct_message_impl (const char * message_id, GMimeStream ** istreams, int qt
 		handle_inline_encoded_data (&retval->mime_part);
 
 	/* cleanup */
-	g_free (messages);
 	return retval;
 }
 
+/***
+****
+***/
 
-GMimeMessage*
-pan_g_mime_parser_construct_message (const char ** raw_texts,
-                                     int qty)
+void
+pan_g_mime_message_mark_read (GMimeMessage * message)
 {
-	int i;
-	int stream_qty;
-	GMimeMessage * retval;
-	GMimeStream ** streams;
+	const char * xref;
+	debug_enter ("pan_g_mime_message_mark_read");
 
-	/* sanity clause */
-	g_return_val_if_fail (raw_texts!=NULL, NULL);
-	g_return_val_if_fail (qty>=1, NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 
-	/* build the GMimeMessages */
-	streams = g_new0 (GMimeStream*, qty);
-	for (stream_qty=i=0; i<qty; ++i) {
-		const char * txt = raw_texts[i];
-		if (is_nonempty_string (txt))
-			streams[stream_qty++] = g_mime_stream_mem_new_with_buffer (txt, strlen(txt));
-	}
-	retval = construct_message_impl ("<foo@bar.mum>", streams, stream_qty);
+	xref = g_mime_message_get_header (message, HEADER_XREF);
 
-	/* cleanup */
-	for (i=0; i<stream_qty; ++i)
-		g_object_unref (streams[i]);
-	g_free (streams);
+	if (is_nonempty_string (xref))
+	{
+		GString * token = g_string_new (NULL);
+		Server * server = serverlist_get_active_server ();
+		const char * message_id_str = g_mime_message_get_message_id (message);
+		const PString message_id = pstring_shallow (message_id_str, -1);
 
-	return retval;
-}
+		/* skip servername */
+		skip_next_token (xref, ' ', &xref);
 
-GMimeMessage*
-pan_g_mime_parser_construct_message_from_file (FILE ** fps,
-                                               int qty)
-{
-	int i;
-	GMimeStream ** streams;
-	GMimeMessage * retval;
+		/* walk through the xrefs group:number group2:number2*/
+		while ((get_next_token_g_str (xref, ' ', &xref, token)))
+		{
+			/* parse out the group and number values */
+			char * delim = strchr (token->str, ':');
+			if (delim != NULL)
+			{
+				/* if the group is subscribed, mark the article as read */
+				const PString group_name = pstring_shallow (token->str, delim-token->str);
+				Group * group = server_get_named_group_in_type (server, &group_name, SERVER_GROUPS_SUBSCRIBED);
+				if (group != NULL)
+				{
+					if (group_ref_articles_if_loaded (group)) {
+						Article * article = group_get_article_by_message_id (group, &message_id);
+						if (article != NULL)
+							articles_set_read_simple (&article, 1, TRUE);
+						group_unref_articles (group, NULL);
+					}
+					else if (group_is_subscribed (group))
+						group_mark_article_read (group, strtoul (delim+1, NULL, 10), TRUE);
 
-	/* sanity clause */
-	g_return_val_if_fail (fps!=NULL, NULL);
-	g_return_val_if_fail (qty>=1, NULL);
-	for (i=0; i<qty; ++i)
-		g_return_val_if_fail (fps[i]!=NULL, NULL);
+				}
+			}
+		}
 
-	/* make buffered streams */
-	streams = g_new0 (GMimeStream*, qty);
-	for (i=0; i<qty; ++i) {
-		GMimeStream * file_stream = g_mime_stream_file_new (fps[i]);
-		streams[i] = g_mime_stream_buffer_new (file_stream, GMIME_STREAM_BUFFER_BLOCK_READ);
-		g_object_unref (file_stream);
+		g_string_free (token, TRUE);
 	}
 
-	/* make the message */
-       	retval = construct_message_impl ("<foo@bar.mum>", streams, qty);
+	debug_exit ("pan_g_mime_message_mark_read");
+}
 
-	/* cleanup */
-	for (i=0; i<qty; ++i)
-		g_object_unref (streams[i]);
-	g_free (streams);
+/**
+ * Loops through the message's Newsgroups: header
+ * until we find a group that is already loaded.
+ * That group is returned.
+ */
+Group*
+pan_g_mime_message_get_group (GMimeMessage * message)
+{
+	Group * retval = NULL;
+	Server * server;
+
+	g_return_val_if_fail (message!=NULL, NULL);
+
+       	server = serverlist_get_active_server ();
+	if (server != NULL)
+	{
+		const char * newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
+		if (is_nonempty_string (newsgroups))
+		{
+			PString group_name;
+
+			for (;;)
+			{
+				int group_name_len;
+				const char * group_name_str;
+				const char * march = newsgroups;
+
+				if (!get_next_token_run (march, ',', &march, &group_name_str, &group_name_len))
+					break;
+
+				group_name = pstring_shallow (group_name_str, group_name_len);
+				retval = server_get_named_group (server, &group_name);
+				if (retval != NULL)
+					break;
+			}
+		}
+	}
 
 	return retval;
 }
 
+/**
+ * Retrieve the charset from a mime message
+ */
+
+static void
+get_charset_partfunc (GMimeObject * obj, gpointer charset_gpointer)
+{
+	GMimePart * part;
+	const GMimeContentType * type;
+	const char ** charset;
+
+	if (!GMIME_IS_PART (obj))
+		return;
+
+	part = GMIME_PART (obj);
+	type = g_mime_object_get_content_type (GMIME_OBJECT (part));
+	charset = (const char **) charset_gpointer;
+	if (g_mime_content_type_is_type (type, "text", "*"))
+		*charset = g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
+}
+
+const char *
+pan_g_mime_message_get_charset (GMimeMessage * message)
+{
+	const char * retval = NULL;
+	g_return_val_if_fail (message!=NULL, NULL);
+
+	g_mime_message_foreach_part (message, get_charset_partfunc, &retval);
+
+	return retval;
+}
