@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __QUEUE_H__
@@ -26,10 +23,9 @@
 #include <glib.h>
 
 #include <pan/task.h>
+#include <pan/base/pstring.h>
 
-
-
-extern void queue_init (int max_sockets, int max_tries);
+extern void queue_init (gboolean online, gboolean remove_failed_tasks);
 
 extern void queue_add (Task*);
 
@@ -39,19 +35,42 @@ extern void queue_set_online (gboolean online);
 
 extern gboolean queue_is_online (void);
 
+enum {
+	QUEUE_MESSAGE_ID_DOWNLOAD	= (1<<0),
+	QUEUE_MESSAGE_ID_SAVE		= (1<<1)
+};
+extern int queue_get_message_id_status (const PString * message_id);
+
 
 /**
 ***  This rest of this file is for the queue manager.
 **/
 
 
-
 /**
 ***  CALLBACKS
 ***
 ***  These callbacks are called when the queue changes.
 **/
 
+/**
+ * @call_obj: GINT_TO_POINTER(running_qty)
+ * @call_arg: GINT_TO_POINTER(total_qty)
+ */
+PanCallback*  queue_get_size_changed_callback (void);
+
+/**
+ * @call_obj: GINT_TO_POINTER(increment)
+ * @call_arg: NULL
+ */
+PanCallback*  queue_get_connection_size_changed_callback (void);
+
+/**
+ * @call_obj: const char ** message_ids
+ * @call_arg: GINT_TO_POINTER(message_id_qty)
+ */
+PanCallback*  queue_get_message_id_status_changed (void);
+
 /**
  * @call_obj: Group**
  * @call_arg: GINT_TO_POINTER(qty)
@@ -76,10 +95,6 @@ extern PanCallback *queue_tasks_removed;
  */
 extern PanCallback *queue_tasks_moved;
 
-extern PanCallback *queue_task_status_changed;
-
-extern PanCallback *queue_max_tries_changed;
-
 /**
 ***  QUEUE MANIPULATION
 **/
@@ -103,9 +118,9 @@ extern void queue_insert_tasks (GSList * tasks, int index);
 
 /**
  * Aborts a running task, has no effect on non-running tasks.
- * @param list of tasks to abort.  queue assumes ownership of the list.
+ * @param list of tasks to stop.  queue assumes ownership of the list.
  */
-extern void queue_abort_tasks (GSList * tasks);
+extern void queue_stop_tasks (GSList * tasks);
 
 /**
  * Remove an item from the queue.  This decreases each task's refcount by 1.
@@ -113,6 +128,8 @@ extern void queue_abort_tasks (GSList * tasks);
  */
 extern void queue_remove_tasks (GSList * tasks);
 
+extern void queue_remove_last_task (void);
+
 /**
  * Reposition an item in the queue.
  * @param tasks the tasks to move
@@ -131,13 +148,9 @@ extern void queue_requeue_failed_tasks (GSList * tasks);
 ***  CONNECTION
 **/
 
-extern int queue_get_max_sockets (void);
+extern gboolean queue_get_remove_failed_tasks (void);
 
-extern void queue_set_max_sockets (int);
-
-extern int queue_get_max_tries (void);
-
-extern void queue_set_max_tries (int);
+extern void queue_set_remove_failed_tasks (gboolean);
 
 extern gboolean queue_has_dedicated_article_thread (void);
 
@@ -154,10 +167,9 @@ typedef enum
 	QUEUE_TASK_STATUS_NOT_QUEUED	= 0,
 	QUEUE_TASK_STATUS_QUEUED	= 1,
 	QUEUE_TASK_STATUS_RUNNING	= 2,
-	QUEUE_TASK_STATUS_FAILED	= 4,
-	QUEUE_TASK_STATUS_ABORTING	= 5,
-	QUEUE_TASK_STATUS_REMOVING	= 6,
-	QUEUE_TASK_STATUS_CONNECTING	= 7
+	QUEUE_TASK_STATUS_STOPPING	= 3,
+	QUEUE_TASK_STATUS_STOPPED	= 4,
+	QUEUE_TASK_STATUS_REMOVING	= 5
 }
 QueueTaskStatus;
 
@@ -169,5 +181,7 @@ extern QueueTaskStatus queue_get_task_status (const Task*);
 
 guint queue_get_running_task_count (void);
 
+char* queue_get_tasks_filename (void);
+
 
 #endif /* __QUEUE_H__ */
