@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,14 +15,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __TASK_HEADERS__H__
 #define __TASK_HEADERS__H__
 
-#include "task.h"
-#include "group.h"
+#include <pan/base/group.h>
+
+#include <pan/task.h>
 
 /**
 ***  TASK_HEADERS CLASS DECLARATION
@@ -47,7 +47,6 @@ struct _TaskHeaders
 
 	/* Fields */
 	Group * group;
-	GPtrArray * headers_needing_bodies;
 	HeaderDownloadType download_type;
 	gboolean download_bodies;
 	int body_index;
